Package: DAMEfinder
Type: Package
Title: Finds DAMEs - Differential Allelicly MEthylated regions
Version: 1.22.0
Authors@R: c(person("Stephany", "Orjuela", role = c("aut", "cre"), 
           email = "sorjuelal@gmail.com", comment = c(ORCID = "0000-0002-1508-461X")),
             person("Dania", "Machlab", role = "aut", email = "dania.machlab@gmail.com"),
             person("Mark", "Robinson", role = "aut", email = "mark.robinson@imls.uzh.ch"))
Description: 'DAMEfinder' offers functionality for taking methtuple or bismark 
        outputs to calculate ASM scores and compute DAMEs. 
        It also offers nice visualization of methyl-circle plots.
License: MIT + file LICENSE
LazyData: FALSE
RoxygenNote: 7.2.2
biocViews: DNAMethylation, DifferentialMethylation, Coverage
Depends: R (>= 4.0)
Imports: stats, Seqinfo, GenomicRanges, IRanges, S4Vectors, readr,
        SummarizedExperiment, GenomicAlignments, stringr, plyr,
        VariantAnnotation, parallel, ggplot2, Rsamtools, BiocGenerics,
        methods, limma, bumphunter, Biostrings, reshape2, cowplot,
        utils
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown, testthat, rtracklayer,
        BSgenome.Hsapiens.UCSC.hg19
BugReports: https://github.com/markrobinsonuzh/DAMEfinder/issues
git_url: https://git.bioconductor.org/packages/DAMEfinder
git_branch: RELEASE_3_22
git_last_commit: a1490db
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:29:07 UTC; biocbuild
Author: Stephany Orjuela [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1508-461X>),
  Dania Machlab [aut],
  Mark Robinson [aut]
Maintainer: Stephany Orjuela <sorjuelal@gmail.com>

# Generated by roxygen2: do not edit by hand

export(calc_asm)
export(calc_derivedasm)
export(dame_track)
export(dame_track_mean)
export(extract_bams)
export(find_dames)
export(get_tstats)
export(methyl_MDS_plot)
export(methyl_circle_plot)
export(methyl_circle_plotCpG)
export(read_tuples)
import(ggplot2)
importFrom(BiocGenerics,"end<-")
importFrom(BiocGenerics,"start<-")
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(GenomicAlignments,readGAlignmentPairs)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,queryHits)
importFrom(Seqinfo,seqnames)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_locate_all)

# DAMEfinder <img src="inst/figures/damefindersticker.png" width="200" align="right" />

[![platforms](http://bioconductor.org/shields/availability/3.11/DAMEfinder.svg)](https://bioconductor.org/packages/devel/bioc/html/DAMEfinder.html#archives)&nbsp;
[![build](http://bioconductor.org/shields/build/devel/bioc/DAMEfinder.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/DAMEfinder/)

**DAMEfinder** (**D**ifferential **A**llele-specific **ME**thylation **finder**) is an R-package that detects allele-specific methylation (ASM) in a cohort of samples, and detects regions of differential ASM within groups of interest, based on **Bisulfite-sequencing** files.

DAMEfinder runs in two modes: **SNP-based** (exhaustive-mode) and **tuple-based** (fast-mode), which converge when calculating differential methylation.

<img src="inst/figures/DAMEfinder_workflow.png" width="520">
Please refer to the vignette for more details on running the pipeline. 

### News:

* Check `NEWS.Rd` [here](https://github.com/markrobinsonuzh/DAMEfinder/blob/master/inst/NEWS.Rd) for latest changes.
* Preprint out [here](https://www.biorxiv.org/content/10.1101/800383v1)
* Published version [here](https://epigeneticsandchromatin.biomedcentral.com/articles/10.1186/s13072-020-00346-8)
* Available on [Bioconductor](https://bioconductor.org/packages/release/bioc/html/DAMEfinder.html)

---

## How do I install it?

A stable version of DAMEfinder is available on Bioconductor.

To install use:

```{r}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("DAMEfinder")
```

Or install directly from the repo:

```{r}
BiocManager::install("markrobinsonuzh/DAMEfinder")

```
---

## What mode should I choose?

It depends on what you want to do and how much time you have. Either way you have to align your reads with [Bismark](https://github.com/FelixKrueger/Bismark) (apologies to other-aligner users).

### SNP-based

To run the **SNP-based** mode you need processed `bam` files *AND* a VCF file for each of your samples with heterozygous SNPs. I know this is typically not the case, so you could alternatively extract heterozygous SNPs using [BisSNP](https://github.com/dnaase/Bis-tools/tree/master/Bis-SNP) (which I have used), or [biscuit](https://github.com/zwdzwd/biscuit).

I call this the "exhaustive-mode" because it extracts an ASM score for every CpG site in the reads containing each SNP from the VCF file. Based on this score DAMEs are detected.

From a biological point of view, you might want to run this mode if you are interested in loss or gain of allele-specificity linked to somatic heterozygous SNPs. More specifically, you could detect genes that exhibit loss of imprinting (e.g. [in colorectal cancer](http://cancerres.aacrjournals.org/content/62/22/6442.long)).

### tuple-based

To run the **tuple-based** mode you have to run [methtuple](https://github.com/PeteHaitch/methtuple) first. The methtuple output is the only thing needed for this mode. 

I call this the fast-mode because you don't need SNP information. The assumption is that intermediate levels of methylation represent ASM along the genome. For example, we have shown that the ASM score can distinguish females from males in the X chromosome. Using SNP information this wouldn't be possible.

---

### What does that look like?

1. **SNP-based** ASM is sequence dependent, therefore to draw ASM I split the reads (horizontal lines) by allele, a bit like this:

<img src="inst/figures/DAME_snp_sampledreads.png" width="550">

This is a (not so great) example of differential SNP-ASM. There is gain of ASM in CR-cancer tissue (CRC2_cimp). The reads shown are a random subset of the total. You could plot all the reads if you like.

The actual ASM scores in this region, as well as the methylation level per allele for several samples looks like this:

<img src="inst/figures/DAME_snp_allsamps.png" width="550">

or summarized by means:

<img src="inst/figures/DAME_snp_allsamps_median.png" width="550">

2. **tuple-based** ASM only relies on the reads.

We can look at the tuple ASM scores in the same region as above, as well as the overall methylation (beta-values) for several samples:

<img src="inst/figures/DAME_tuple_allsamps_median.png" width="550">

---

## Feedback

If you have problems installing, running, or just have questions regarding the package or theory behind it, please open an issue and I will answer as soon as possible. 

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(cache = 0, fig.width = 6, 
                      fig.height = 7)

## ----style, echo = FALSE, results = 'asis'------------------------------------
BiocStyle::markdown()

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("DAMEfinder")

## -----------------------------------------------------------------------------
suppressPackageStartupMessages({
  library(DAMEfinder)
  library(SummarizedExperiment)
  library(GenomicRanges)
  library(BSgenome.Hsapiens.UCSC.hg19)
  })

bam_files <- c(system.file("extdata", "NORM1_chr19_trim.bam", 
                           package = "DAMEfinder"),
               system.file("extdata", "CRC1_chr19_trim.bam", 
                           package = "DAMEfinder"))

vcf_files <- c(system.file("extdata", "NORM1.chr19.trim.vcf", 
                           package = "DAMEfinder"),
               system.file("extdata", "CRC1.chr19.trim.vcf", 
                           package = "DAMEfinder"))

sample_names <- c("NORM1", "CRC1")

#Use another reference file for demonstration, and fix the seqnames
genome <- BSgenome.Hsapiens.UCSC.hg19
seqnames(genome) <- gsub("chr","",seqnames(genome))
reference_file <- DNAStringSet(genome[[19]], use.names = TRUE)
names(reference_file) <- 19

#Extract reads and extract methylation according to allele
snp.list <- extract_bams(bam_files, vcf_files, sample_names, reference_file,
                       coverage = 2)

#CpG sites for first SNP in VCF file from sample NORM1
snp.list$NORM1[[1]]

#CpG sites for first SNP in VCF file from sample CRC1
snp.list$CRC1[[1]]


## -----------------------------------------------------------------------------

derASM <- calc_derivedasm(snp.list)

derASM
assays(derASM)

## -----------------------------------------------------------------------------
x <- assay(derASM, "der.ASM")
head(x)

## -----------------------------------------------------------------------------

data(extractbams_output)

#The data loaded is an output from `split_bams()`, therefore we run 
#`calc_derivedasm` to get the SummarizedExperiment
derASM <- calc_derivedasm(extractbams_output, cores = 1, verbose = FALSE)

#We remove all CpG sites with any NA values, but not 0s
filt <- rowSums(!is.na(assay(derASM, "der.ASM"))) == 8 
derASM <- derASM[filt,]

#set the design matrix
grp <- factor(c(rep("CRC",4),rep("NORM",4)), levels = c("NORM", "CRC"))
mod <- model.matrix(~grp)
mod

#Run default
dames <- find_dames(derASM, mod)

head(dames)

#Run empirical method
dames <- find_dames(derASM, mod, pvalAssign = "empirical")

head(dames)


## -----------------------------------------------------------------------------
tuple_files <- c(system.file("extdata", "NORM1_chr19.qs.CG.2.tsv.gz", 
                             package = "DAMEfinder"),
                 system.file("extdata", "CRC1_chr19.qs.CG.2.tsv.gz", 
                             package = "DAMEfinder"))

sample_names <- c("NORM1", "CRC1")

tuple_list <- read_tuples(tuple_files, sample_names)

head(tuple_list$NORM1)

## -----------------------------------------------------------------------------

ASM_mat <- calc_asm(tuple_list)
ASM_mat

## -----------------------------------------------------------------------------
#load package data
data(readtuples_output)

#run calc_asm and filter object
ASMscore <- calc_asm(readtuples_output)
filt <- rowSums(!is.na(assay(ASMscore, "asm"))) == 5 #filt to avoid warnings
ASMscore <- ASMscore[filt,]

#make design matrix (or specify a contrast)
grp <- factor(c(rep("CRC",3),rep("NORM",2)), levels = c("NORM", "CRC"))
mod <- model.matrix(~grp)

#run default and increase maxGap to get longer, more sparse regions
dames <- find_dames(ASMscore, mod, maxGap = 300)

head(dames)

#run alternative mode
dames <- find_dames(ASMscore, mod,  maxGap = 300, pvalAssign = "empirical")

head(dames)


## ----dametrack----------------------------------------------------------------

#Here I will use the tuple-ASM SummExp
colData(ASMscore)$group <- grp
colData(ASMscore)$samples <- colnames(ASMscore)

#Set a DAME as a GRanges. I choose a one from the tables we obtained above
dame <- GRanges(19,IRanges(323736,324622))

dame_track(dame = dame,
           ASM = ASMscore)

## ----dt2----------------------------------------------------------------------
dame_track(dame = dame,
           ASM = ASMscore,
           window = 2)


## ----dt3----------------------------------------------------------------------

dame <- GRanges(19,IRanges(387966,387983))

grp <- factor(c(rep("CRC",4),rep("NORM",4)), levels = c("NORM", "CRC"))
colData(derASM)$group <- grp

dame_track(dame = dame,
           derASM = derASM)


## ----dt4----------------------------------------------------------------------
dame_track(dame = dame,
           derASM = derASM,
           plotSNP = TRUE)


## ----dt5----------------------------------------------------------------------
dame_track(dame = dame,
           derASM = derASM,
           ASM = ASMscore)


## ----dt6----------------------------------------------------------------------
dame_track_mean(dame = dame,
           derASM = derASM,
           ASM = ASMscore)


## ----fig1---------------------------------------------------------------------
#put SNP in GRanges (you can find the SNP with the dame_track function)
snp <- GRanges(19, IRanges(267039, width = 1)) #always set the width if your 
#GRanges has 1 site

snp

bam.file <- system.file("extdata", "CRC1_chr19_trim.bam", 
                        package = "DAMEfinder")

vcf.file <- system.file("extdata", "CRC1.chr19.trim.vcf", 
                        package = "DAMEfinder")

methyl_circle_plot(snp = snp, vcfFile = vcf.file, bamFile = bam.file, 
                   refFile = reference_file)

## ----fig2---------------------------------------------------------------------

cpgsite <- GRanges(19, IRanges(266998, width = 1))

methyl_circle_plot(snp = snp, vcfFile = vcf.file, bamFile = bam.file, 
                   refFile = reference_file, cpgsite = cpgsite)

## ----fig3---------------------------------------------------------------------

cpgsite <- GRanges(19, IRanges(266998, width = 1))

methyl_circle_plotCpG(cpgsite = cpgsite, bamFile = bam.file, 
                      refFile = reference_file)

## ----fig4---------------------------------------------------------------------

#a random region
dame <- GRanges(19, IRanges(266998,267100))

methyl_circle_plot(snp = snp, vcfFile = vcf.file, bamFile = bam.file, 
                   refFile = reference_file, dame = dame)


## ----fig5---------------------------------------------------------------------

grp <- factor(c(rep("CRC",3),rep("NORM",2)), levels = c("NORM", "CRC"))
methyl_MDS_plot(ASMscore, group = grp)


## -----------------------------------------------------------------------------
utils::sessionInfo()

#' Plot means per group of score tracks.
#'
#'
#' @param dame GRanges object containing a region of interest, or detected with
#'   find_dames
#' @param window Number of CpG sites outside (up or down-stream) of the DAME
#'   should be plotted. Default = 0.
#' @param positions Number of bp sites outside (up or down-stream) of the DAME
#'   should be plotted. Default = 0.
#' @param derASM SummarizedExperiment object obtained from calc_derivedasm
#'   (Filtering should be done by the user)
#' @param ASM SummarizedExperiment object obtained from calc_asm (Filtering
#'   should be done by the user)
#' @param colvec Vector of colors (mainly useful for the SNP plot, because I add
#'   it with cowplot, so I don't export a ggplot, optional)
#'
#' @return Plot
#'
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#' @importFrom BiocGenerics start
#' @importFrom BiocGenerics end
#' @importFrom Seqinfo seqnames
#' @importFrom SummarizedExperiment assay
#' @importFrom SummarizedExperiment colData
#' @importFrom S4Vectors queryHits
#' @importFrom BiocGenerics start<-
#' @importFrom BiocGenerics end<-
#' @import ggplot2
#'
#' @examples 
#' library(GenomicRanges)
#' DAME <- GRanges(19, IRanges(306443,310272))
#' data('readtuples_output')
#' ASM <- calc_asm(readtuples_output)
#' SummarizedExperiment::colData(ASM)$group <- c(rep('CRC',3),rep('NORM',2))
#' SummarizedExperiment::colData(ASM)$samples <- colnames(ASM)
#' dame_track_mean(dame = DAME, 
#'                 ASM = ASM)
#' 
#' @export

dame_track_mean <- function(dame, window = 0, positions = 0, 
    derASM = NULL, ASM = NULL, colvec = NULL) {
    
    res_dame <- dame
    start(res_dame) <- start(dame) - positions
    end(res_dame) <- end(dame) + positions
    
    summarize_dat <- function(cond, SumExp, scorename, subtab, 
        pos) {
        idx <- colData(SumExp)$group == cond
        medians <- rowMeans(as.matrix(subtab)[, 
            idx], na.rm = TRUE)
        mins <- medians - (apply((as.matrix(subtab)[, idx]), 
            1, FUN = sd_rem))
        maxs <- medians + (apply((as.matrix(subtab)[, idx]), 
            1, FUN = sd_rem))
        return(data.frame(Means = medians, lower = mins, upper = maxs, 
            Group = cond, Score = scorename, pos = pos))
    }
    
    # custom se function
    sd_rem <- function(v) {
        if (sum(is.na(v)) == length(v)) {
            return(NA)
            stop()
        }
        v <- stats::na.omit(v)
        return(sqrt(stats::var(v)/length(v)))
    }
    
    if (!is.null(derASM)) {
        
        if (!all.equal(colData(derASM)$samples, colnames(derASM))) {
            stop("Sample names in colData() and colnames are different")
        }
        
        ASMsnp <- assay(derASM, "der.ASM")
        ref <- assay(derASM, "ref.meth")/assay(derASM, "ref.cov")
        alt <- assay(derASM, "alt.meth")/assay(derASM, "alt.cov")
        
        snpgr <- SummarizedExperiment::rowRanges(derASM)
        
        over <- GenomicRanges::findOverlaps(snpgr, res_dame)
        if (window != 0) {
            win <- c(seq(from = (queryHits(over)[1] - window), 
                to = (queryHits(over)[1] - 1), by = 1), queryHits(over), 
                seq(from = (utils::tail(queryHits(over), n = 1) + 1), 
                to = (utils::tail(queryHits(over), n = 1) + window), by = 1))
        } else {
            win <- queryHits(over)
        }
        
        # ASMsnp
        subASMsnp <- as.data.frame(ASMsnp[win, ])
        subref <- as.data.frame(ref[win, ])
        subalt <- as.data.frame(alt[win, ])
        substart <- start(snpgr)[win]
        
        ASMsnp_meds <- lapply(unique(colData(derASM)$group), 
            summarize_dat, SumExp = derASM, scorename = "ASMsnp", 
            subtab = subASMsnp, pos = substart)
        
        ref_meds <- lapply(unique(colData(derASM)$group), summarize_dat, 
            SumExp = derASM, scorename = "REF:meth", subtab = subref, 
            pos = substart)
        
        alt_meds <- lapply(unique(colData(derASM)$group), summarize_dat, 
            SumExp = derASM, scorename = "ALT:meth", subtab = subalt, 
            pos = substart)
        
        ASMsnp_meds_full <- do.call(rbind, ASMsnp_meds)
        ref_meds_full <- do.call(rbind, ref_meds)
        alt_meds_full <- do.call(rbind, alt_meds)
        
    }
    
    if (!is.null(ASM)) {
        
        if (!all.equal(colData(ASM)$samples, colnames(ASM))) {
            stop("Sample names in colData() and colnames are different")
        }
        
        meth <- (assay(ASM, "MM") + assay(ASM, "MU") + assay(ASM, 
            "UM"))/assay(ASM, "cov")
        
        ASMtuple <- assay(ASM, "asm")
        tupgr <- SummarizedExperiment::rowRanges(ASM)
        
        # ASMtuple
        over <- GenomicRanges::findOverlaps(tupgr, res_dame)
        
        if (window != 0) {
            win <- c(seq(from = (queryHits(over)[1] - window), 
                to = (queryHits(over)[1] - 1), by = 1), queryHits(over), 
                seq(from = (utils::tail(queryHits(over), n = 1) + 1), 
                to = (utils::tail(queryHits(over), n = 1) + window), by = 1))
        } else {
            win <- queryHits(over)
        }
        
        subASMtuple <- as.data.frame(ASMtuple[win, ])
        submeth <- as.data.frame(meth[win, ])
        subtupger <- tupgr$midpt[win]
        
        ASMtuple_meds <- lapply(unique(colData(ASM)$group), summarize_dat, 
            SumExp = ASM, scorename = "ASMtuple", subtab = subASMtuple, 
            pos = subtupger)
        
        ASMtuple_meds_full <- do.call(rbind, ASMtuple_meds)
        
        meth_meds <- lapply(unique(colData(ASM)$group), summarize_dat, 
            SumExp = ASM, scorename = "meth", subtab = submeth, 
            pos = subtupger)
        
        meth_meds_full <- do.call(rbind, meth_meds)
    }
    
    
    if ((!is.null(derASM)) & (!is.null(ASM))) {
        message("Using both scores")
        full_long <- rbind(ASMtuple_meds_full, meth_meds_full, 
            ASMsnp_meds_full, ref_meds_full, alt_meds_full)
        full_long$Score <- factor(full_long$Score, levels = c("ASMtuple", 
            "meth", "ASMsnp", "REF:meth", "ALT:meth"))
    }
    
    if (is.null(ASM) & !is.null(derASM)) {
        message("Using ASMsnp score")
        full_long <- rbind(ASMsnp_meds_full, ref_meds_full, alt_meds_full)
        full_long$Score <- factor(full_long$Score, levels = c("ASMsnp", 
            "REF:meth", "ALT:meth"))
    }
    
    if (is.null(derASM) & !is.null(ASM)) {
        message("Using ASMtuple score")
        full_long <- rbind(ASMtuple_meds_full, meth_meds_full)
        
    }
    
    # To draw rectangle on region
    forect <- data.frame(xmin = start(dame), xmax = end(dame), 
        ymin = 0, ymax = Inf)
    
    # plot scores
    m1 <- ggplot(data = full_long) + 
      geom_line(mapping = aes(x = .data$pos, y = .data$Means, color = .data$Group)) +
      geom_ribbon(mapping = aes(x = .data$pos,
                                 ymin = .data$lower,
                                 ymax = .data$upper,
                                 fill = .data$Group), alpha = 0.2) + 
        geom_rect(data = forect, aes(xmin = .data$xmin, xmax = .data$xmax,
                                      ymin = .data$ymin, ymax = .data$ymax),
                  alpha = 0.1) +
      facet_grid(Score ~ ., scales = "free_y") +
      theme_bw() +
      xlab("position") + 
      ggtitle("Summary scores")
    
    if (!is.null(colvec)) {
        p <- m1 + scale_color_manual(values = colvec) + 
            scale_fill_manual(values = colvec)
    } else {
        p <- m1
    }
    
    return(p)
}

context("test-dame_track_median")
library(GenomicRanges)


DAME <- GRanges(19, IRanges(306443,310272)) 
data(readtuples_output)
ASM <- calc_asm(readtuples_output)
SummarizedExperiment::colData(ASM)$group <- c(rep("CRC",3), rep("NORM", 2))
SummarizedExperiment::colData(ASM)$samples <- colnames(ASM)

test_that("dame track with ASM", {
  p <- dame_track_mean(dame = DAME, ASM = ASM)#derASM = derASM, )
  expect_true(is.ggplot(p))
})

library(SummarizedExperiment)


DATA_PATH_DIR <- "inst/extdata/moredata"

get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)

tuple_files <- list.files(DATA_PATH_DIR, ".tsv.gz")
tuple_files <- get_data_path(tuple_files)

readtuples_output <- read_tuples(tuple_files, 
                                 c("CRC1", "CRC2", "CRC3", "NORM1", "NORM3"))


usethis::use_data(readtuples_output, overwrite = TRUE, compress = 'xz')

context("test-tupleasm_dames")

library(SummarizedExperiment)


DATA_PATH_DIR <- system.file("extdata", ".", package = "DAMEfinder")

get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)

tuple_files <- list.files(DATA_PATH_DIR, ".tsv.gz")
tuple_files <- get_data_path(tuple_files)

data(readtuples_output)
grp <- factor(c(rep("CRC",3),rep("NORM",2)), levels = c("NORM", "CRC"))
mod <- model.matrix(~grp)

test_that("end to end read_tuples", {
  ASM <- read_tuples(tuple_files, c("CRC1", "NORM1"))
  expect_is(ASM, "list")
  expect_length(ASM, 2)
})


ASMscore <- calc_asm(readtuples_output)
filt <- rowSums(!is.na(assay(ASMscore, "asm"))) == 5 #filt to avoid warnings
ASMscore <- ASMscore[filt,]

test_that("end to end calc_asm", {
  #ASMscore <- calc_asm(readtuples_output)
  expect_s4_class(ASMscore, "RangedSummarizedExperiment")
  expect_length(colnames(ASMscore), 5)
  expect_length(assays(ASMscore), 6)
})

# test_that("end to end tstat calc", {
#   ASMt <- get_tstats(ASMscore, mod, maxGap = 300, verbose = FALSE)
#   expect_s4_class(ASMt, "RangedSummarizedExperiment")
# })


test_that("end to end find_dames", {
  dames <- find_dames(ASMscore, mod, verbose = FALSE)
  expect_is(dames, "data.frame")
})

test_that("use permutations", {
  dames <- find_dames(ASMscore, mod, pvalAssign = "empirical", verbose = FALSE)
  expect_is(dames, "data.frame")
})

context("split_bams")
#This is a long test

DATA_PATH_DIR <- system.file("extdata", ".", package = "DAMEfinder")

get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)


bam_files <- get_data_path("NORM1_chr19_trim.bam")

vcf_files <- get_data_path("NORM1.chr19.trim.vcf")

sample_names <- "NORM1"

#This fasta is from ftp://ftp.ensembl.org/pub/grch37/release-91/fasta/homo_sapiens/dna/
#reference_file <- get_data_path("19.fa")

#Get reference file 
library(BSgenome.Hsapiens.UCSC.hg19)
genome <- BSgenome.Hsapiens.UCSC.hg19
seqnames(genome) <- gsub("chr","",seqnames(genome))
dna <- DNAStringSet(genome[[19]], use.names = TRUE)
names(dna) <- 19


test_that("output is GRangesList", {
  GRanges_list <- extract_bams(bam_files, vcf_files, sample_names, dna)
  expect_s4_class(GRanges_list$NORM1[[1]], "GRanges")
})

#' Draw methylation circle plot
#'
#' Draws CpG site methylation status as points, in reads containing a specific
#' SNP. Generates one plot per bam file.
#'
#' @param snp GRanges object containing SNP location.
#' @param cpgsite (optional) GRanges object containing a single CpG site
#'   location of interest.
#' @param vcfFile vcf file.
#' @param bamFile bismark bam file path.
#' @param refFile fasta reference file path. Or \code{DNAStringSet} with DNA
#'  sequence.
#' @param build genome build used. default = "hg19"
#' @param dame (optional) GRanges object containing a region to plot.
#' @param letterSize Size of alleles drawn in plot. Default = 2.5.
#' @param pointSize Size of methylation circles. Default = 3.
#' @param sampleName FIX?: this is to save the vcf file to not generate it every
#'   time you run the function.
#' @param sampleReads Whether a subset of reads should be plotted.
#'   Default = FALSE.
#' @param numReads Number of reads to plot per allele, if sampleReads is TRUE.
#'   Default = 20
#'
#' @return Plot
#' @examples
#' DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
#'
#' get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
#' bam_files <- get_data_path('NORM1_chr19_trim.bam')
#' vcf_files <- get_data_path('NORM1.chr19.trim.vcf')
#' sample_names <- 'NORM1'
#' #reference_file
#' suppressPackageStartupMessages({library(BSgenome.Hsapiens.UCSC.hg19)})
#' genome <- BSgenome.Hsapiens.UCSC.hg19
#' seqnames(genome) <- gsub("chr","",seqnames(genome))
#' dna <- DNAStringSet(genome[[19]], use.names = TRUE)
#' names(dna) <- 19
#'
#' snp <- GenomicRanges::GRanges(19, IRanges::IRanges(292082, width = 1))
#' methyl_circle_plot(snp = snp,
#'  vcfFile = vcf_files,
#'  bamFile = bam_files,
#'  refFile = dna,
#'  sampleName = sample_names)
#'
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#' @importFrom BiocGenerics start
#' @importFrom BiocGenerics end
#' @importFrom Seqinfo seqnames
#' @import ggplot2
#'
#' @export

methyl_circle_plot <- function(snp, vcfFile, bamFile, refFile, build = "hg19",
    dame = NULL, letterSize = 2.5, pointSize = 3, sampleName = "sample1", 
    cpgsite = NULL, sampleReads = FALSE, numReads = 20) {
    
    message("Reading vcf file")
    if (!file.exists(paste0(sampleName, ".RData"))) {
      param <- VariantAnnotation::ScanVcfParam(fixed="ALT", info=NA, geno=NA)  
      vcf <- VariantAnnotation::readVcf(vcfFile, genome = build, 
                                        param = param)
        save(vcf, file = paste0(sampleName, ".RData"))
    } else {
        load(paste0(sampleName, ".RData"))
    }
    
    snp.loc <- which(start(vcf) == start(snp) &  
                       as.character(seqnames(vcf)) == levels(seqnames(snp)))
    if (length(snp.loc) == 0) {
        message("Sample does not contain SNP")
        return(NULL)
        stop()
    }
    ref <- as.character(VariantAnnotation::ref(vcf))[snp.loc]
    alt <- as.character(
      BiocGenerics::unlist(VariantAnnotation::alt(vcf)))[snp.loc]
    
    message("Getting reads")
    alns.pairs <- GenomicAlignments::readGAlignmentPairs(bamFile, 
        param = Rsamtools::ScanBamParam(tag = c("MD", "XM", "XR", 
            "XG"), which = snp), use.names = TRUE)
    
    alns <- unlist(alns.pairs)  #unpaired
    
    split <- splitReads(alns, ref, snp)
    
    alt.reads <- split$alt.reads
    ref.reads <- split$ref.reads
    
    if (sampleReads) {
        alt.reads <- sample(alt.reads, numReads)
        ref.reads <- sample(ref.reads, numReads)
    }
    
    message("Sorting reads")
    # don't unlist, paired
    alt.pairs <- alns.pairs[names(alns.pairs) %in% alt.reads]
    ref.pairs <- alns.pairs[names(alns.pairs) %in% ref.reads]
    alns.pairs <- c(alt.pairs, ref.pairs)  #I just do this to sort them
    
    #### get reference and CpG
    #### positions####------------------------------------
    
    # Get limits for plotting
    
    if (is.null(dame)) {
        left <- min(start(alns))
        right <- max(end(alns))
        window <- GRanges(seqnames(snp), IRanges(left, right))
    } else {
        window <- dame
        left <- start(dame)
        right <- end(dame)
    }
    
    message("Reading reference")
    # open reference seq to get correct CpG locations within that
    # reference chunk
    if (typeof(refFile) == "character") {
        fa <- open(Rsamtools::FaFile(refFile, index = paste0(refFile, 
            ".fai")))
        dna <- Rsamtools::scanFa(fa, param = window)
    } else {
        dna <- refFile[window]
    }
    
    dna <- as.character(dna)
    cgsite <- stringr::str_locate_all(dna, "CG")[[1]][, 1]  #also look at GpCs?
    
    if (length(cgsite) < 1) {
        return(NULL)
        stop("No CpG sites associated to this SNP")
    }
    
    mepos <- cgsite/Biostrings::nchar(dna)  #location of CpG sites
    
    # Use MD tag from bam to extract methylation status for each
    # site detected above
    message("Getting meth state per read-pair")
    conversion <- vapply(alns.pairs, function(x) {
        
        # change C locations for G locations if reference context is
        # different
        if (S4Vectors::mcols(x)$XG[1] == "GA") {
            cgsite <- cgsite + 1
        }
        
        # fuse pair info in one row
        conversion <- matrix(NA, 1, length(cgsite))
        
        # a <- mcols(x)$MD
        read.start <- start(x) - left + 1  #start of read
        read.end <- end(x) - left + 1  #end
        
        for (pair in c(1, 2)) {
            
            something <- mcols(x[pair])$MD
            tag <- getMD(something)
            MDtag <- tag$MDtag
            nucl.num <- tag$nucl.num
            
            # Get the locations from the specific read: Count along the
            # numerical MDtag until you reach the actual CpG start
            count <- 0
            this.mepos <- cgsite - read.start[pair] + 1
            
            # Fill in conversion table with meth state for each read 21
            # is unmethylated 19 is methylated 0 is not in read
            
            for (p in seq_along(this.mepos)) {
                if (is.na(conversion[1, p])) {
                    if (cgsite[p] > read.end[pair] || cgsite[p] < 
                    read.start[pair]) {
                    conversion[1, p] <- NA
                    next
                    } else {
                    for (i in seq_along(nucl.num)) {
                        count <- count + nucl.num[i]
                        if (count >= this.mepos[p]) {
                        count <- 0
                        break
                        }
                    }
                    
                    # if there is a mismatch, the MDtag shows what the base is 
                    # in the ref. So, if it shows a C or G, it means that the 
                    # ref is C and the read has T or A, therefore it is 
                    # unmethylated (converted)
                    if (MDtag[i] %in% c("C", "G")) {
                        conversion[1, p] <- 21
                    } else {
                        conversion[1, p] <- 19
                    }
                        }
                } else {
                    next
                }
            }
        }
        return(conversion)
    }, double(length(cgsite)))
    
    # Remove reads without CpG sites
    rem <- colSums(is.na(conversion)) >= dim(conversion)[1]
    conversion <- conversion[, !rem]
    
    alt.reads <- alt.reads[alt.reads %in% colnames(conversion)]
    ref.reads <- ref.reads[ref.reads %in% colnames(conversion)]
    alns.pairs <- alns.pairs[names(alns.pairs) %in% colnames(conversion)]
    
    #### Get snp position in window
    #### ####----------------------------------------
    
    letter <- c(rep(alt, length(alt.reads)), rep(ref, length(ref.reads)))
    
    snp.start <- start(snp) - left + 1
    
    # and cpg of interest
    if (!is.null(cpgsite)) {
        cpg.start <- start(cpgsite) - left + 1
    } else {
        cpg.start = snp.start
    }
    
    #### plot
    #### ####-------------------------------------------------------------
    
    message("Plotting")
    
    # data for points
    d <- data.frame(CpG = rep(cgsite, length(alns.pairs)), 
        read = rep(seq(from = 1, 
        to = length(alns.pairs), by = 1), each = length(cgsite)), 
        value = as.vector(conversion), stringsAsFactors = FALSE)
    
    # data for segments
    reads <- d$read
    
    xstart <- vapply(reads, function(x) {
        newd <- d[which(d$read == x), ]
        vals <- which(!is.na(newd$value))
        min(newd$CpG[vals], snp.start)
    }, FUN.VALUE = double(1))
    
    xend <- vapply(reads, function(x) {
        newd <- d[which(d$read == x), ]
        vals <- which(!is.na(newd$value))
        max(newd$CpG[vals], snp.start)
    }, FUN.VALUE = double(1))
    
    d2 <- unique(data.frame(xstart, xend, reads, stringsAsFactors = FALSE))
    d2$snp <- c(rep("a", length(alt.reads)), rep("r", length(ref.reads)))
    
    # To manually scale the colors
    cols <- c("#0E4071", "#d55e00", "#0E4071", "#d55e00")
    names(cols) <- c("a", "r", alt, ref)
    
    mp <- ggplot() + 
        scale_shape_identity() + 
        theme_void() + 
        geom_segment(data = d2, aes(x = .data$xstart, 
                                    y = .data$reads, 
                                    xend = .data$xend, 
                                    yend = .data$reads,
                                    colour = .data$snp), linewidth = 0.2) + 
        geom_point(data = d, aes(x = .data$CpG,
                                 y = .data$read,
                                 shape = .data$value), 
            fill = "white", size = pointSize) + 
        geom_point(aes(x = snp.start, y = seq(from = 1, to = length(alns.pairs),
            by = 1), shape = letter, colour = letter), size = letterSize) + 
        geom_point(aes(x = cpg.start, y = 0), shape = 24, size = 3, 
            fill = "green") + 
        scale_color_manual(values = cols) + 
        guides(color = FALSE) + 
        ggtitle(sampleName)
    
    return(mp)
}

#' Draw methylation circle plot without SNP
#'
#' Draws CpG site methylation status as points, in reads containing a specific
#' CpG site. Generates one plot per bam file.
#'
#' @param cpgsite GRanges object containing a single CpG site location of
#'   interest
#' @param bamFile bismark bam file path
#' @param refFile fasta reference file path
#' @param pointSize Size of methylation circles. Default = 3.
#' @param dame (optional) GRanges object containing a region to plot
#' @param order Whether reads should be sorted by methylation status. Default=
#' False.
#' @param sampleName Plot title.
#' @param sampleReads Whether a subset of reads should be plotted.
#'   Default = FALSE.
#' @param numReads Number of reads to plot, if sampleReads is TRUE. Default = 20
#' @return Plot
#' @examples
#' DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
#' get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
#' bam_files <- get_data_path('NORM1_chr19_trim.bam')
#' sample_names <- 'NORM1'
#' #reference_file
#' suppressPackageStartupMessages({library(BSgenome.Hsapiens.UCSC.hg19)})
#' genome <- BSgenome.Hsapiens.UCSC.hg19
#' seqnames(genome) <- gsub("chr","",seqnames(genome))
#' dna <- DNAStringSet(genome[[19]], use.names = TRUE)
#' names(dna) <- 19
#'
#' cpg <- GenomicRanges::GRanges(19, IRanges::IRanges(292082, width = 1))
#' methyl_circle_plotCpG(cpgsite = cpg,
#'  bamFile = bam_files,
#'  refFile = dna)
#'
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#' @importFrom BiocGenerics start
#' @importFrom BiocGenerics end
#' @importFrom Seqinfo seqnames
#' @import ggplot2
#'
#' @export
methyl_circle_plotCpG <- function(cpgsite = cpgsite, bamFile = bamFile, 
    pointSize = 3, refFile = refFile, dame = NULL, order = FALSE, 
    sampleName = NULL, sampleReads = FALSE, numReads = 20) {
    
    alns.pairs <- GenomicAlignments::readGAlignmentPairs(bamFile, 
        param = Rsamtools::ScanBamParam(tag = c("MD", "XM", "XR", 
            "XG"), which = cpgsite), use.names = TRUE)
    
    
    if (sampleReads) {
        ran.names <- sample(names(alns.pairs), numReads)
        alns.pairs <- alns.pairs[names(alns.pairs) %in% ran.names]
    }
    
    alns <- unlist(alns.pairs)
    
    #### get reference and CpG
    #### positions####-------------------------------------
    
    # Get limits for plotting
    
    if (is.null(dame)) {
        left <- min(start(alns))
        right <- max(end(alns))
        window <- GRanges(seqnames(cpgsite), IRanges(left, right))
    } else {
        window <- dame
        left <- start(dame)
        right <- end(dame)
    }
    
    message("Reading reference")
    # open reference seq to get correct CpG locations within that
    # reference chunk
    if (typeof(refFile) == "character") {
        fa <- open(Rsamtools::FaFile(refFile, index = paste0(refFile, 
            ".fai")))
        dna <- Rsamtools::scanFa(fa, param = window)
    } else {
        dna <- refFile[window]
    }
    dna <- as.character(dna)
    cgsite <- stringr::str_locate_all(dna, "CG")[[1]][, 1]  #also look at GpCs?
    
    if (length(cgsite) < 1) {
        stop("No CpG sites in these reads")
    }
    
    mepos <- cgsite/Biostrings::nchar(dna)  #location of CpG sites
    
    ##### Use MD tag from bam to extract methylation status for each
    ##### site detected above ####----------------------------
    message("Getting meth state per read-pair")
    conversion <- vapply(alns.pairs, function(x) {
        
        # change C locations for G locations if reference context is
        # different
        if (S4Vectors::mcols(x)$XG[1] == "GA") {
            cgsite <- cgsite + 1
        }
        
        # fuse pair info in one row
        conversion <- matrix(NA, 1, length(cgsite))
        
        # a <- mcols(x)$MD
        read.start <- start(x) - left + 1  #start of read
        read.end <- end(x) - left + 1  #end
        
        for (pair in c(1, 2)) {
            
            something <- mcols(x[pair])$MD
            tag <- getMD(something)
            MDtag <- tag$MDtag
            nucl.num <- tag$nucl.num
            
            # Get the locations from the specific read: Count along the
            # numerical MDtag until you reach the actual CpG start
            count <- 0
            this.mepos <- cgsite - read.start[pair] + 1
            
            # Fill in conversion table with meth state for each read 21
            # is unmethylated 19 is methylated 0 is not in read
            
            for (p in seq_along(this.mepos)) {
                if (is.na(conversion[1, p])) {
                    if (cgsite[p] > read.end[pair] || cgsite[p] < 
                    read.start[pair]) {
                    conversion[1, p] <- NA
                    next
                    } else {
                    for (i in seq_along(nucl.num)) {
                        count <- count + nucl.num[i]
                        if (count >= this.mepos[p]) {
                            count <- 0
                            break
                        }
                    }
                    
                    # if there is a mismatch, the MDtag shows what the base is 
                    # in the ref. So, if it shows a C or G, it means that the 
                    # ref is C and the read has T or A, therefore it is 
                    # unmethylated (converted)
                    if (MDtag[i] %in% c("C", "G")) {
                        conversion[1, p] <- 21
                    } else {
                        conversion[1, p] <- 19
                    }
                    }
                } else {
                    next
                }
            }
        }
        return(conversion)
    }, double(length(cgsite)))
    
    # Remove reads without CpG sites
    rem <- colSums(is.na(conversion)) >= dim(conversion)[1]
    conversion <- conversion[, !rem]
    
    alns.pairs <- alns.pairs[names(alns.pairs) %in% colnames(conversion)]
    
    #### Get CpG of interest in window
    #### ####------------------------------------
    
    cpg.start <- start(cpgsite) - left + 1
    
    #### plot
    #### ####-------------------------------------------------------------
    
    message("Plotting")
    
    # data for points
    d <- data.frame(CpG = rep(cgsite, length(alns.pairs)), 
        read = rep(seq(from = 1, 
        to = length(alns.pairs), by = 1), each = length(cgsite)), 
        value = as.vector(conversion))
    
    # reorder reads to plot
    if (isTRUE(order)) {
        trans <- ifelse(d$value == 19, 1, 0)
        und <- unique(d$read)
        sums_per_read <- vapply(und, function(i) sum(trans[d$read == 
            i], na.rm = TRUE))
        und_ord <- und[order(sums_per_read)]
        
        d$order <- 0
        for (i in und) {
            d$order[d$read == i] <- which(und_ord == i)
        }
        d$read <- d$order
    }
    
    # data for segments
    reads <- d$read
    
    xstart <- vapply(reads, function(x) {
        newd <- d[which(d$read == x), ]
        vals <- which(!is.na(newd$value))
        min(newd$CpG[vals])
    }, FUN.VALUE = double(1))
    
    xend <- vapply(reads, function(x) {
        newd <- d[which(d$read == x), ]
        vals <- which(!is.na(newd$value))
        max(newd$CpG[vals])
    }, FUN.VALUE = double(1))
    
    d2 <- unique(data.frame(xstart, xend, reads))
    
    ggplot() + 
        scale_shape_identity() + 
        theme_void() + 
        geom_segment(data = d2, aes(x = .data$xstart,
                                    y = .data$reads,
                                    xend = .data$xend,
                                    yend = .data$reads),
                     colour = "grey", linewidth = 0.5) + 
        geom_point(data = d, aes(x = .data$CpG,
                                 y = .data$read,
                                 shape = .data$value), 
            fill = "white", size = pointSize) + 
        geom_point(aes(x = cpg.start, y = 0), shape = 24, size = 3, 
            fill = "green") + 
        guides(color = "none") + 
        ggtitle(sampleName)
    
}

context("test-methyl_circle_plot")
library(GenomicRanges)

DATA_PATH_DIR <- system.file("extdata", ".", package = "DAMEfinder")

get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)


bam_files <- get_data_path("NORM1_chr19_trim.bam")

vcf_files <- get_data_path("NORM1.chr19.trim.vcf")

sample_names <- "NORM1"

#reference_file <- get_data_path("19.fa")
#Get reference file 
library(BSgenome.Hsapiens.UCSC.hg19)
genome <- BSgenome.Hsapiens.UCSC.hg19
seqnames(genome) <- gsub("chr","",seqnames(genome))
reference_file <- DNAStringSet(genome[[19]], use.names = TRUE)
names(reference_file) <- 19

snp <- GRanges(19, IRanges(292082, width = 1))
#cpgsite <- GRanges(19, IRanges(387982, width = 1))


test_that("end to end methyl plot", {
  
  p <- methyl_circle_plot(snp = snp,
                          vcfFile = vcf_files,
                          bamFile = bam_files,
                          refFile = reference_file,
                          letterSize = 3,
                          #dame = dame,
                          sampleName = sample_names,
                          pointSize = 2)
  expect_true(is.ggplot(p))
  
  
  #cleanup
  if(file.exists("NORM1.RData")) file.remove("NORM1.RData")
})

test_that("end to end methyl_circle_plotCpG", {
  p <- methyl_circle_plotCpG(cpgsite = snp,
                        bamFile = bam_files,
                        refFile = reference_file,
                        pointSize = 2)
  expect_true(is.ggplot(p))
})

#' Calculate SNP-based ASM
#'
#' Combines all the \code{GRangeslist} generated in \code{\link{extract_bams}}
#' into a \code{\link{RangedSummarizedExperiment}} object, and calculates
#' SNP-based allele-specific methylation. 
#'
#' @param sampleList List of samples returned from \code{\link{extract_bams}}.
#' @param cores Number of cores to thread.
#' @param verbose If the function should be verbose.
#' @return \code{RangedSummarizedExperiment} containing in assays:
#'
#'   - der.ASM: matrix with SNP-based ASM 
#'   - snp.table: Matrix with SNP associated to the CpG site. 
#'   - ref.cov: Coverage of the 'reference' allele. 
#'   - alt.cov: Coevarage of the 'alternative' allele. 
#'   - ref.meth: Methylated reads from the 'reference' allele. 
#'   - alt.meth: Methylated reads from the 'alternative' allele.
#' @md
#'
#' @examples
#' data(extractbams_output)
#' derASM <- calc_derivedasm(extractbams_output[c(1,2)], cores = 1, 
#'    verbose = FALSE)
#' 
#' @importFrom S4Vectors mcols
#' @importFrom S4Vectors mcols<-
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#'
#' @export

calc_derivedasm <- function(sampleList, cores = 1, verbose = TRUE) {
    
    # filter duplicated sites and choose the one with the highest
    # meth.diff
    allGR <- parallel::mclapply(sampleList, function(snp.table) {
        if (verbose) 
            message(".", appendLF = FALSE)
        
        # remove NULL values from list
        snp.table <- plyr::compact(snp.table)
        
        # Create a (actual) GRangesList, to be able to unlist
        GRlist <- GenomicRanges::GRangesList(snp.table)
        w <- unlist(GRlist)
        
        # calculate meth.diff (aka derASM)
        prop.alt <- mcols(w)$meth.alt/mcols(w)$cov.alt
        prop.ref <- mcols(w)$meth.ref/mcols(w)$cov.ref
        mcols(w)$meth.diff <- abs(prop.alt - prop.ref)
        
        # transform to keys
        full.keys <- paste0(seqnames(w), ".", start(w))
        
        # get unique ranges to loop
        unique.keys <- unique(sort(full.keys))
        
        # For duplicate sites, choose the one with the highest
        # meth.diff
        cols <- vapply(unique.keys, function(x) {
            # cols <- vapply(unGR, function(x){
            sHits <- as.integer(match(x, full.keys))
            
            if (length(sHits) <= 1) {
                return(methods::as(mcols(w)[sHits, ], "matrix"))
            } else {
                methdiffs <- mcols(w)$meth.diff[sHits]
                high <- as.integer(which.max(methdiffs))
                return(methods::as(mcols(w)[sHits[high], ], "matrix"))
            }
        }, character(6))
        
        # the vapply returns a tranversed matrix with everything as
        # characters, so I have to transform this.
        mcol <- data.frame(as.numeric(cols[1, ]), 
                           as.numeric(cols[2, ]), 
                           as.numeric(cols[3, ]), 
                           as.numeric(cols[4, ]), 
                           cols[5, ], 
                           as.numeric(cols[6, ]), stringsAsFactors = FALSE)
        
        colnames(mcol) <- colnames(mcols(w))
        ss <- limma::strsplit2(unique.keys, ".", fixed = TRUE)
        
        # return ordered GRanges
        unGR <- GRanges(ss[, 1], IRanges(as.integer(ss[, 2]), 
            width = 1))
        mcols(unGR) <- mcol
        
        # human ordering
        unGR <- Seqinfo::sortSeqlevels(unGR)
        unGR <- sort(unGR)
        
        # The unGR has the filtered sites for this sample
        return(unGR)
    }, mc.cores = cores)
    
    # Try to extract from all the GRanges the unique sites by
    # generating a key
    all_keys <- lapply(allGR, function(u) {
        paste0(seqnames(u), ".", start(u))
    })
    
    key <- unique(unlist(all_keys))
    ss <- limma::strsplit2(key, ".", fixed = TRUE)
    keyGR <- GRanges(ss[, 1], IRanges(as.numeric(ss[, 2]), width = 1))
    
    # get matrix of scores across all samples
    if (verbose) 
        message("Summarizing scores")
    trueASM_table <- mapply(function(df, k) {
        m <- match(key, k)
        mcols(df)$meth.diff[m]
    }, allGR, all_keys)
    
    rownames(trueASM_table) <- key
    
    # Get matrices of coverages
    if (verbose) 
        message("Summarizing coverages")
    ref.cov_table <- mapply(function(df, k) {
        m <- match(key, k)
        return(mcols(df)$cov.ref[m])
    }, allGR, all_keys)
    
    alt.cov_table <- mapply(function(df, k) {
        m <- match(key, k)
        return(mcols(df)$cov.alt[m])
    }, allGR, all_keys)
    
    ref.meth_table <- mapply(function(df, k) {
        m <- match(key, k)
        return(mcols(df)$meth.ref[m])
    }, allGR, all_keys)
    
    alt.meth_table <- mapply(function(df, k) {
        m <- match(key, k)
        return(mcols(df)$meth.alt[m])
    }, allGR, all_keys)
    
    #Calculate stat ASM (prop test style)
    prop <- (ref.meth_table + alt.meth_table) / 
      (ref.cov_table + alt.cov_table)
    
    sigma <- sqrt(prop * (1 - prop) * ((1/ref.cov_table) + 
                                         (1/alt.cov_table)))
    
    zstat <- ((ref.meth_table/ref.cov_table) - 
                        (alt.meth_table/alt.cov_table)) / sigma
    
    # Get matrix for snp ID
    if (verbose) 
        message("Summarizing SNP info")
    snp_match_table <- mapply(function(df, k) {
        m <- match(key, k)
        mcols(df)$snp[m]
    }, allGR, all_keys)
    
    # length(snp_match_table)
    if (dim(snp_match_table)[1] != dim(trueASM_table)[1]) {
        stop("Tables contain different sizes")
    }
    
    if (dim(snp_match_table)[1] != dim(trueASM_table)[1]) {
        stop("Tables contain different sizes")
    } else {
        if (verbose) 
            message(sprintf("Returning %i CpG sites for %i samples", 
                dim(trueASM_table)[1], dim(trueASM_table)[2]))
    }
    
    ## Put all matrices into S4 vector
    derived_ASM_matrix <- SummarizedExperiment::SummarizedExperiment(
        assays = S4Vectors::SimpleList(
        der.ASM = trueASM_table, 
        z.ASM = abs(zstat),                               
        snp.table = snp_match_table, 
        ref.cov = ref.cov_table, 
        alt.cov = alt.cov_table, 
        ref.meth = ref.meth_table, 
        alt.meth = alt.meth_table), 
        rowRanges = keyGR, 
        colData = S4Vectors::DataFrame(samples = names(allGR)))
    
    # last human sorting
    derived_ASM_matrix <- Seqinfo::sortSeqlevels(derived_ASM_matrix)
    derived_ASM_matrix <- sort(derived_ASM_matrix)
    
    return(derived_ASM_matrix)
}

context("test-derivedASM_dames")
library(SummarizedExperiment)
library(limma)

data(extractbams_output)
grp <- factor(c(rep("CRC",4),rep("NORM",4)), levels = c("NORM", "CRC"))
mod <- model.matrix(~grp)

derASM <- calc_derivedasm(extractbams_output, cores = 1, verbose = FALSE)
filt <- rowSums(!is.na(assay(derASM, "der.ASM"))) == 8 #filt to avoid warnings
derASM <- derASM[filt,]

test_that("end to end calc_derivedasm", {
  expect_s4_class(derASM, "RangedSummarizedExperiment")
  expect_length(colnames(derASM), 8)
})


test_that("all assays created", {
  expect_length(assays(derASM), 7)
})

# test_that("end to end tstat calc", {
#   derASMt <- get_tstats(derASM, mod, verbose = FALSE)
#   expect_s4_class(derASMt, "RangedSummarizedExperiment")
#   expect_s4_class(rowData(derASMt), "DataFrame")
# })

test_that("end to end find_dames", {
  dames <- find_dames(derASM, mod, verbose = FALSE)
  expect_is(dames, "data.frame")
})

context("test-dame_track")
library(GenomicRanges)

data(extractbams_output)

derASM <- calc_derivedasm(extractbams_output, cores = 1, verbose = TRUE)
DAME <- GRanges(19, IRanges(306443,310272))

SummarizedExperiment::colData(derASM)$group <- c(rep("CRC",4), rep("NORM", 4))

test_that("SNP dame track", {
  p <- dame_track(dame = DAME, derASM = derASM)
  expect_true(is.ggplot(p))
})

data(readtuples_output)
ASM <- calc_asm(readtuples_output)
SummarizedExperiment::colData(ASM)$group <- c(rep("CRC",3), rep("NORM", 2))
SummarizedExperiment::colData(ASM)$samples <- colnames(ASM)

test_that("dame track with both", {
  p <- dame_track(dame = DAME, derASM = derASM, ASM = ASM)
  expect_true(is.ggplot(p))
})

test_that("other options", {
  p <- dame_track(dame = DAME, derASM = derASM, ASM = ASM,
                  plotSNP = TRUE)
  expect_true(is.ggplot(p))
})

#' Calculate ASM Score
#'
#' This function takes in a list of samples resulting from the read_tuples
#' function and returns a SummarizedExperiment of Allele-Specific Methylation
#' (ASM) scores, where each row is a tuple and each column is a sample.
#'
#' Calculates ASM score for a list of samples in the output format of the result
#' of read_tuples This functions uses the following other functions: process,
#' calcScore, calcWeight.
#'
#' @param sampleList List of samples returned from \code{\link{read_tuples}}
#' @param beta The beta parameter used to calculate the weight in the ASM score.
#'   \code{link{calc_weight}} uses this parameter to penalize fully methylated
#'   or unmethylated tuples. Default = 0.5.
#' @param verbose If the function should be verbose. Default = TRUE.
#' @param a The distance from 0.5 allowed, where 0.5 is a perfect MM:UU balance
#'   for a tuple. In the default mode this value is set to 0.2, and we account
#'   for the instances where the balance is between 0.3 and 0.7.
#' @param transform Transform the calculated tuple ASM scores. We use the
#'   modulus square root function which outputs the square root, while
#'   preserving the original sign.
#' @param coverage Remove tuples with total reads below coverage. Default
#'   = 5.
#'
#' @return A \code{SummarizedExperiment} of ASM scores where the rows are all
#'   the tuples and the columns the sample names.
#' @examples
#' DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
#' get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
#'
#' tuple_files <- list.files(DATA_PATH_DIR, '.tsv.gz')
#' tuple_files <- get_data_path(tuple_files)
#' ASM <- read_tuples(tuple_files, c('CRC1', 'NORM1'))
#' ASMscore <- calc_asm(ASM)
#'
#' @export
#'
calc_asm <- function(sampleList, beta = 0.5, a = 0.2, transform = modulus_sqrt, 
    coverage = 5, verbose = TRUE) {
    
    if (!is.vector(sampleList)) 
        stop("Input is not a list of more than one tibble")
    
    if (verbose) 
        message("Calculating log odds.")
    sampleList <- lapply(sampleList, calc_logodds)
    
    if (verbose) 
        message("Calculating ASM score: ", appendLF = FALSE)
    sampleList <- lapply(sampleList, function(u) {
        if (verbose) 
            message(".", appendLF = FALSE)
        calc_score(u, beta = beta, a = a)
    })
    if (verbose) 
        message(" done.")
    
    if (verbose) 
        message("Creating position pair keys: ", appendLF = FALSE)
    # get key of unique tuples
    all_keys <- lapply(sampleList, function(u) {
        if (verbose) 
            message(".", appendLF = FALSE)
        paste0(u$chr, ".", u$pos1, ".", u$pos2)
    })
    key <- unique(unlist(all_keys))
    if (verbose) 
        message(" done.")
    
    # get matrix of ASM scores across all samples
    if (verbose) 
        message("Assembling table: ", appendLF = FALSE)
    asm <- mapply(function(df, k) {
        if (verbose) 
            message(".", appendLF = FALSE)
        m <- match(key, k)
        df$asm_score[m]
    }, sampleList, all_keys)
    
    rownames(asm) <- key
    colnames(asm) <- names(sampleList)
    if (verbose) 
        message(" done.")
    
    if (verbose) 
        message("Transforming.")
    asm <- transform(asm)
    
    # get matrix of coverage, and tuple methylation
    if (verbose) 
        message("Assembling coverage tables: ", appendLF = FALSE)
    tot.coverage <- mapply(function(df, k) {
        if (verbose) 
            message(".", appendLF = FALSE)
        m <- match(key, k)
        df$cov[m]
    }, sampleList, all_keys)
    
    
    # MM
    MM <- mapply(function(df, k) {
        if (verbose) 
            message(".", appendLF = FALSE)
        m <- match(key, k)
        df$MM[m]
    }, sampleList, all_keys)
    
    # MU
    MU <- mapply(function(df, k) {
        if (verbose) 
            message(".", appendLF = FALSE)
        m <- match(key, k)
        df$MU[m]
    }, sampleList, all_keys)
    
    # UM
    UM <- mapply(function(df, k) {
        if (verbose) 
            message(".", appendLF = FALSE)
        m <- match(key, k)
        df$UM[m]
    }, sampleList, all_keys)
    
    # UU
    UU <- mapply(function(df, k) {
        if (verbose) 
            message(".", appendLF = FALSE)
        m <- match(key, k)
        df$UU[m]
    }, sampleList, all_keys)
    
    # Get ranges
    ss <- limma::strsplit2(rownames(asm), ".", fixed = TRUE)
    gr <- GenomicRanges::GRanges(ss[, 1], IRanges::IRanges(as.numeric(ss[, 
        2]), as.numeric(ss[, 3])))
    names(gr) <- rownames(asm)
    gr$midpt <- floor((GenomicRanges::start(gr) + GenomicRanges::end(gr))/2)
    
    # Build object
    sa <- SummarizedExperiment::SummarizedExperiment(
        assays = S4Vectors::SimpleList(asm = asm, 
        cov = tot.coverage, MM = MM, MU = MU, UM = UM, UU = UU), 
        rowRanges = gr)
    
    # filter by coverage
    
    filt <- rowSums(!is.na(SummarizedExperiment::assay(sa, "cov")) & 
        SummarizedExperiment::assay(sa, "cov") >= coverage) == 
        ncol(sa)
    sa <- sa[filt, ]
    gr <- gr[filt]
    
    if (verbose) 
        message("Returning SummarizedExperiment with ", nrow(asm), 
            " CpG pairs", appendLF = FALSE)
    o <- order(Seqinfo::seqnames(sa), gr$midpt)
    sa[o]
}



#' Calculate score
#'
#' This function calculates the ASM score for every tuple in a given sample. The
#' ASM score is a multiplication of the log odds ratio by a weight that reflects
#' the extent of allele-specific methylation. This weight is obtained with the
#' \code{\link{calc_weight}} function.
#'
#' @param beta parameter for the \code{calc_weight} function. It's the alpha and
#'   beta values for the Beta function.
#' @param a parameter for the calc_weight function. The weight will be the
#'   probability that the MM/(MM+UU) ratio lies between 0.5-a and 0.5+a.
#' @param df data frame of a sample containing all information per tuple
#'   (MM,UU,UM and MU counts, as well as the log odds ratio per tuple) needed
#'   for the ASM score.
#' @details This function returns an allele-specific methylation (ASM) score for
#'   every given tuple in a sample. The ASM score is a product of the log odds
#'   ratio and a weight reflecting a measure of allele-specificity using the MM
#'   and UU counts.
#'
#'
#' @return The same object with an additional column for the ASM score.
#'
calc_score <- function(df, beta = 0.5, a = 0.2) {
    weights <- calc_weight(df$MM, df$UU, beta = beta, a = a)
    df$asm_score <- df$logodds * weights
    df
}

#' Calculate the log odds ratio
#'
#' This function calculates the log odds ratio for a CpG tuple:
#' \code{(MM*UU)/(UM*MU)}, where 'M' stands for methylated and 'U' for
#' unmethylated. 'MM' reflects the count for instances the CpG pair is
#' methylated at both positions. The higher the MM and UU counts for that CpG
#' pair, the higher the log odds ratio.
#'
#'
#' @param s A data frame that contains the MM,UU,UM, and MU counts for each CpG
#'   tuple for a particular sample. It is the resulting object of the
#'   \code{read_tuples}.
#' @param eps Count added to each of the MM,UU,UM and MU counts to avoid
#'   dividing by zero for example. The default is set to 1.
#'
#' @return The same object is returned with an additional column for the log
#'   odds ratio.
calc_logodds <- function(s, eps = 1) {
    MM <- UU <- MU <- UM <- NULL
    ratio <- with(s, ((MM + eps) * (UU + eps))/((MU + eps) * 
        (UM + eps)))
    s$logodds <- log10(ratio)
    s
}


#' Get Modulus Square Root
#'
#' Function to calculate signed square root (aka modulus square root).
#'
#' @param values Vector or matrix of ASM scores where each column is a sample.
#'   These values are transformed with a square root transformation that
#'   (doesn't) preserve the sign.
#'
#' @return Vector or matrix of transformed scores.
#'
modulus_sqrt <- function(values) {
    # sign(values)*sqrt(abs(values))
    sqrt(abs(values))
}


#' Calculate Weight for ASM Score
#'
#' This function calculates a weight which reflects MM to UU balance, where M
#' stands for methylated and U for unmethylated. Given the MM and UU counts for
#' a particular tuple, the weight is obtained using the \code{link{pbeta}}
#' function.
#'
#'
#' @param beta parameter for the beta distribution. In B(alpha,beta), we set
#'   alpha=beta=0.5 by default.
#' @param MM The read counts for where pos1 and pos2 of the tuple were both
#'   methylated.
#' @param UU The read counts for where pos1 and pos2 of the tuple were both
#'   unmethylated.
#' @param a parameter for how far from 0.5 we go as a measure of allele-specific
#'   methylation. The weight is the probability that the MM:(MM+UU) ratio is
#'   between 0.5-a and 0.5+a. The default is set to 0.2.
#'
#' @details For a given tuple with MM and UU counts, the weight that reflects
#' allele-scpecificity is calculated as follows:
#'
#' - Prior:\deqn{p(\theta|\alpha,\beta) \sim Beta(\alpha,\beta),} where
#' \eqn{\theta = \frac{MM}{MM+UU}} and \eqn{\alpha = \beta = 0.5}.
#' \eqn{p(\theta|\alpha,\beta)} represents our prior belief which is that
#' tuples are either fully methylated or fully unmethylated, rather than
#' allele-specifically methylated which is a much rarer event.
#' - Likelihood: \deqn{p(x|\alpha,\beta) \propto \theta^{MM}(1-\theta)^{UU},}
#' where x is our observation (the MM and UU counts).
#' - Posterior:\deqn{p(\theta|x) \propto p(x|\theta)*p(\theta|\alpha,\beta)}
#' \deqn{p(\theta|x) \propto \theta^{MM-0.5}(1-\theta)^{UU-0.5},}
#' where \eqn{\alpha = \beta = 0.5}. This posterior also follows a beta
#' distribution \eqn{\sim Beta(\alpha'=MM+0.5, \beta'=UU+0.5)}
#' @md
#'
#' @return A number that reflects allele-specificity given MM and UU counts for
#'   a CpG pair. This is used as a weight that is multiplied by the log odds
#'   ratio to give the final ASM score of that tuple.
#'
#' #calc_weight(MM=50, UU=50)
#' #0.9999716
#'
#' #calc_weight(MM=20, UU=60)
#' #0.1646916
#'
#' 
#'
calc_weight <- function(MM, UU, beta = 0.5, a = 0.2) {
    s1 <- beta + MM
    s2 <- beta + UU
    stats::pbeta(0.5 + a, shape1 = s1, shape2 = s2) - stats::pbeta(0.5 - 
        a, shape1 = s1, shape2 = s2)
}

context("test-mdsplot")

data(readtuples_output)
ASM <- calc_asm(readtuples_output)

test_that("end to end mdsplot", {
  p <- methyl_MDS_plot(ASM, group = c(rep("CRC",3), rep("NORM", 2)))
  expect_true(is.ggplot(p))
})

#' Detect allele-specific methylation from a bam file
#'
#' The function takes a bam (from bismark) and vcf file for each sample. For
#' each SNP contained in the vcfile it calculates the proportion of methylated
#' reads for each CpG site at each allele. At the end it returns (saves to
#' working directory) a GRanges list, where each GRanges contains all the CpG
#' sites overlapping the reads containing a specific SNP.
#'
#' @param bamFiles List of bam files.
#' @param vcfFiles List of vcf files.
#' @param sampleNames Names of files in the list.
#' @param referenceFile fasta file used to generate the bam files. Or 
#'   \code{DNAStringSet} with DNA sequence.
#' @param coverage Minimum number of reads covering a CpG site on each allele.
#'   Default = 2.
#' @param cores Number of cores to use. See package {parallel} for description
#'   of core. Default = 1.
#' @param verbose Default = TRUE
#'
#' @return A list of GRanges for each sample. Each list is saved in a separate
#'   .rds file.
#' @examples
#' DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
#' get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
#' bamFiles <- get_data_path('NORM1_chr19_trim.bam')
#' vcfFiles <- get_data_path('NORM1.chr19.trim.vcf')
#' sampleNames <- 'NORM1'
#' 
#' #referenceFile 
#' suppressPackageStartupMessages({library(BSgenome.Hsapiens.UCSC.hg19)})
#' genome <- BSgenome.Hsapiens.UCSC.hg19
#' seqnames(genome) <- gsub("chr","",seqnames(genome))
#' dna <- DNAStringSet(genome[[19]], use.names = TRUE)
#' names(dna) <- 19
#'
#' GRanges_list <- extract_bams(bamFiles, vcfFiles, sampleNames, dna)
#'
#' @importFrom BiocGenerics start
#' @importFrom BiocGenerics end
#' @importFrom S4Vectors mcols
#' @importFrom S4Vectors mcols<-
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#' @importFrom GenomicAlignments readGAlignmentPairs
#' @importFrom Seqinfo seqnames
#'
#' @export
extract_bams <- function(bamFiles, vcfFiles, sampleNames, referenceFile, 
  coverage = 4, cores = 1, verbose = TRUE) {
    
    if (verbose) 
        message("Reading reference file", appendLF = TRUE)
    if (typeof(referenceFile) == "character") {
        fa <- open(Rsamtools::FaFile(referenceFile, 
            index = paste0(referenceFile, ".fai")))
        
    }
    
    # get names and indeces per sample to apply to
    sample_list <- seq(from = 1, to = length(sampleNames), by = 1)
    names(sample_list) <- sampleNames
    
    lapply(sample_list, function(samp) {
        
        message(sprintf("Running sample %s", sampleNames[samp]))
        
        if (verbose) 
            message("Reading VCF file", appendLF = TRUE)
        param <- VariantAnnotation::ScanVcfParam(fixed="ALT", info=NA, geno=NA)  
        vcf <- VariantAnnotation::readVcf(vcfFiles[samp], 
              param = param)
        bam.file <- bamFiles[samp]
        
        # message(sprintf('Processing chromosome %s',chrom))
        if (verbose) 
            message("Extracting methylation per SNP", appendLF = TRUE)
        snp.table <- parallel::mcmapply(function(t, u, v) {
          
          #t = vcf[, 1] = CHROM 
          #u = vcf[, 2] = POS 
          #v = vcf[, 4] = REF
            
            # Applying to a GRanges takes too long so I create one each
            # time
            snp <- GRanges(seqnames = t, IRanges(start = as.integer(u), 
                width = 1))
            
            chrom <- t
            
            params <- Rsamtools::ScanBamParam(tag = c("MD", "XM", 
                "XR", "XG"), which = snp)
            
            alns.pairs <- GenomicAlignments::readGAlignmentPairs(bam.file, use.names = TRUE, 
                param = params)
            
            alns <- unlist(alns.pairs)  #unpaired
            split <- splitReads(alns, v, snp)
            
            alt.reads <- split$alt.reads
            ref.reads <- split$ref.reads
            
            if (length(ref.reads) <= 1 | length(alt.reads) <= 
                1) {
                if (verbose) 
                    message("0 or 1 read in one or both alleles", 
                    appendLF = TRUE)
                return(NULL)
                # next
            }
            
            #### get reference and CpG
            #### positions####-----------------------------------
            
            # Get limits for calling methylation and grabing reference
            # sequence
            left <- min(start(alns))
            right <- max(end(alns))
            window <- GRanges(seqnames(snp), IRanges(left, 
                right))
            
            if (typeof(referenceFile) == "S4") {
                dna <- referenceFile[window]
                
            } else if (typeof(referenceFile) == "character") {
                # open reference seq to get correct CpG locations within that
                # reference chunk
                dna <- Rsamtools::scanFa(fa, param = window)
            }
            
            dna <- as.character(dna)
            cgsite <- stringr::str_locate_all(dna, "CG")[[1]][,1]  
            #also look at GpCs?
            
            if (length(cgsite) < 1) {
                if (verbose) 
                  message("No CpG sites associated to this SNP", 
                      appendLF = TRUE)
                return(NULL)
                # next
            }
            mepos <- cgsite/Biostrings::nchar(dna)  #location of CpG sites
            
            ##### Use MD tag from bam to extract methylation status for each
            ##### site detected above ####----------------------------
            conversion <- vapply(alns.pairs, function(x) {
                # change C locations for G locations if reference context is
                # different
                XGcondition <- mcols(x)$XG[1]
                if (XGcondition == "GA") {
                  cgsite <- cgsite + 1
                }
                
                # fuse pair info in one row
                conversion <- matrix(NA, 1, length(cgsite))
                
                read.start <- start(x) - left + 1  #start of read
                read.end <- end(x) - left + 1  #end
                
                for (pair in c(1, 2)) {
                  something <- mcols(x[pair])$MD
                  tag <- getMD(something)
                  MDtag <- tag$MDtag
                  nucl.num <- tag$nucl.num
                  
                  # Get the locations from the specific read: Count along the
                  # numerical MDtag until you reach the actual CpG start Fill
                  # in conversion table with meth state for each read 1 is
                  # unmethylated 2 is methylated NA is not in read
                  
                  count <- 0
                  this.mepos <- cgsite - read.start[pair] + 1
                  
                  for (p in seq_along(this.mepos)) {
                    if (is.na(conversion[1, p])) {
                      if (cgsite[p] > read.end[pair] || cgsite[p] < 
                        read.start[pair]) {
                        conversion[1, p] <- NA
                        next
                      } else {
                        for (i in seq_along(nucl.num)) {
                          count <- count + nucl.num[i]
                          if (count >= this.mepos[p]) {
                            count <- 0
                            break
                          }
                        }
                        
                # if there is a mismatch, the MDtag shows what the base is in
                # the ref. So, if it shows a C or G, it means that the ref is
                # C or G, and the read has T or A, therefore it is
                # unmethylated (converted)
                        if (MDtag[i] %in% c("C", "G")) {
                          conversion[1, p] <- 1
                        } else {
                          conversion[1, p] <- 2
                        }
                      }
                    } else {
                      next
                    }
                  }
                }
                return(conversion)
            }, double(length(cgsite)))
            
            # Return methylated reads and total coverage for each allele
            
            if (is.null(dim(conversion))) {
                ref.conv <- conversion[names(conversion) %in% 
                  ref.reads]
                alt.conv <- conversion[names(conversion) %in% 
                  alt.reads]
                
                ref.cov <- sum(!is.na(ref.conv))
                alt.cov <- sum(!is.na(alt.conv))
                
                ref.meth <- sum(!is.na(ref.conv) & ref.conv == 
                  2)
                alt.meth <- sum(!is.na(alt.conv) & alt.conv == 
                  2)
                
            } else {
                ref.conv <- conversion[, colnames(conversion) %in% 
                  ref.reads]
                alt.conv <- conversion[, colnames(conversion) %in% 
                  alt.reads]
                
                ref.cov <- rowSums(!is.na(ref.conv))
                alt.cov <- rowSums(!is.na(alt.conv))
                
                ref.meth <- rowSums(!is.na(ref.conv) & 
                  ref.conv == 2)
                alt.meth <- rowSums(!is.na(alt.conv) & 
                  alt.conv == 2)
            }
            
            # Build GRanges
            GR <- GRanges(gsub("chr", "", chrom), IRanges(start = left + 
                cgsite, width = 1))
            mcols(GR)$cov.ref <- ref.cov
            mcols(GR)$cov.alt <- alt.cov
            mcols(GR)$meth.ref <- ref.meth
            mcols(GR)$meth.alt <- alt.meth
            mcols(GR)$snp <- paste0(seqnames(snp), 
                ".", start(snp))
            
            # Keep CpG sites where each allele has at least 'coverage'
            # reads.
            filt <- rowSums(cbind(GR$cov.ref, GR$cov.alt) >= 
                coverage) >= 2
            if (sum(filt) < 2) {
                if (verbose) 
                  message("No CpG sites sufficiently covered at this SNP", 
                    appendLF = TRUE)
                return(NULL)
            }
            
            gr <- GR[filt]
            return(gr)
            
        #}, t = vcf[, 1], u = vcf[, 2], v = vcf[, 4], SIMPLIFY = FALSE, 
        }, t = as.character(seqnames(vcf)), 
           u = start(vcf), 
           v = as.character(VariantAnnotation::ref(vcf)), SIMPLIFY = FALSE, 
            USE.NAMES = FALSE, mc.cores = cores)
        
        message(sprintf("Done with sample %s", sampleNames[samp]))
        return(snp.table)
    })
}

library(DAMEfinder)

DATA_PATH_DIR <- "inst/extdata"

get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)

bam_files <- vapply(c("CRC1_chr19_trim.bam",
                      "moredata/CRC2_chr19_trim.bam",
                      "moredata/CRC3_chr19_trim.bam",
                      "moredata/CRC4_chr19_trim.bam",
                      "NORM1_chr19_trim.bam",
                      "moredata/NORM2_chr19_trim.bam",
                      "moredata/NORM3_chr19_trim.bam",
                      "moredata/NORM4_chr19_trim.bam"),get_data_path,
                    USE.NAMES = FALSE)

vcf_files <- vapply(c("CRC1.chr19.trim.vcf",
                      "moredata/CRC2.chr19.trim.vcf",
                      "moredata/CRC3.chr19.trim.vcf",
                      "moredata/CRC4.chr19.trim.vcf",
                      "NORM1.chr19.trim.vcf",
                      "moredata/NORM2.chr19.trim.vcf",
                      "moredata/NORM3.chr19.trim.vcf",
                      "moredata/NORM4.chr19.trim.vcf"),get_data_path,
                    USE.NAMES = FALSE)

sample_names <- c("CRC1","CRC2","CRC3","CRC4","NORM1","NORM2","NORM3","NORM4")

reference_file <- get_data_path("19.fa")

extractbams_output <- extract_bams(bam_files, vcf_files, sample_names, 
                                 reference_file)

usethis::use_data(extractbams_output, overwrite = TRUE, compress = 'xz')

#' Plot score tracks
#'
#'
#' @param dame GRanges object containing a region of interest, or detected with
#'   find_dames
#' @param window Number of CpG sites outside (up or down-stream) of the DAME
#'   should be plotted. Default = 0.
#' @param positions Number of bp sites outside (up or down-stream) of the DAME
#'   should be plotted. Default = 0.
#' @param derASM SummarizedExperiment object obtained from calc_derivedasm
#'   (Filtering should be done by the user)
#' @param ASM SummarizedExperiment object obtained from calc_asm (Filtering
#'   should be done by the user)
#' @param colvec Vector of colors (mainly useful for the SNP plot, because I add
#'   it with cowplot, so I don't export a ggplot, optional)
#' @param plotSNP whether to add the SNP track, only if derASM is specified.
#'   Default = FALSE
#'
#' @return Plot
#'
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#' @importFrom BiocGenerics start
#' @importFrom BiocGenerics end
#' @importFrom Seqinfo seqnames
#' @importFrom SummarizedExperiment assay
#' @importFrom SummarizedExperiment colData
#' @importFrom S4Vectors queryHits
#' @importFrom BiocGenerics start<-
#' @importFrom BiocGenerics end<-
#' @import ggplot2
#' 
#' @examples 
#' library(GenomicRanges)
#' DAME <- GRanges(19, IRanges(306443,310272))
#' data('readtuples_output')
#' ASM <- calc_asm(readtuples_output)
#' SummarizedExperiment::colData(ASM)$group <- c(rep('CRC',3),rep('NORM',2))
#' SummarizedExperiment::colData(ASM)$samples <- colnames(ASM)
#' dame_track(dame = DAME, 
#'            ASM = ASM) 
#'
#' @export

dame_track <- function(dame, window = 0, positions = 0, derASM = NULL, 
    ASM = NULL, colvec = NULL, plotSNP = FALSE) {
    
    res_dame <- dame
    start(res_dame) <- start(dame) - positions
    end(res_dame) <- end(dame) + positions
    
    if (!is.null(derASM)) {
        
        if (!all.equal(colData(derASM)$samples, colnames(derASM))) {
            stop("Sample names in colData() and colnames are different")
        }
        
        ASMsnp <- assay(derASM, "der.ASM")
        SNP <- assay(derASM, "snp.table")
        ref <- assay(derASM, "ref.meth")/assay(derASM, "ref.cov")
        alt <- assay(derASM, "alt.meth")/assay(derASM, "alt.cov")
        
        snpgr <- SummarizedExperiment::rowRanges(derASM)
        
        over <- GenomicRanges::findOverlaps(snpgr, res_dame)
        if (window != 0) {
            win <- c(seq(from = (queryHits(over)[1] - window), 
                to = (queryHits(over)[1] - 1), by = 1), queryHits(over), 
                seq(from = (utils::tail(queryHits(over), n = 1) + 
                    1), to = (utils::tail(queryHits(over), n = 1) + 
                    window), by = 1))
        } else {
            win <- queryHits(over)
        }
        
        # ASMsnp
        subASMsnp <- as.data.frame(ASMsnp[win, ])
        subref <- as.data.frame(ref[win, ])
        subalt <- as.data.frame(alt[win, ])
        
        subASMsnp$pos <- subref$pos <- subalt$pos <- start(snpgr)[win]
        subASMsnp_long <- reshape2::melt(subASMsnp, id.vars = "pos", 
            measure.vars = colnames(ASMsnp))
        subASMsnp_long$score <- "ASMsnp"
        
        # marg.meth per allele
        subref_long <- reshape2::melt(subref, id.vars = "pos", 
            measure.vars = colnames(ASMsnp))
        subref_long$score <- "REF:meth"
        
        subalt_long <- reshape2::melt(subalt, id.vars = "pos", 
            measure.vars = colnames(ASMsnp))
        subalt_long$score <- "ALT:meth"
        
        # SNP
        subSNP <- SNP[win, ]
        subSNP[is.na(subSNP)] <- "none"
        subSNP <- data.frame(subSNP, stringsAsFactors = FALSE)
        subSNP$pos <- start(snpgr)[win]
        subSNP_long <- reshape2::melt(subSNP, id.vars = "pos", 
            measure.vars = colnames(ASMsnp), value.name = "snp.pos")
        
        # Make SNP name nicer
        loc <- as.integer(stringr::str_extract(subSNP_long$snp.pos, 
            "[0-9]+$"))
        chrom <- as.integer(stringr::str_extract(subSNP_long$snp.pos, 
            "^[0-9]+"))
        subSNP_long$snp.pos <- ifelse(subSNP_long$snp.pos == 
            "none", "none", sprintf("chr%s:%s", chrom, loc))
    }
    
    if (!is.null(ASM)) {
        
        if (!all.equal(colData(ASM)$samples, colnames(ASM))) {
            stop("Sample names in colData() and colnames are different")
        }
        
        meth <- (assay(ASM, "MM") + assay(ASM, "MU") + assay(ASM, 
            "UM"))/assay(ASM, "cov")
        
        ASMtuple <- assay(ASM, "asm")
        tupgr <- SummarizedExperiment::rowRanges(ASM)
        
        
        # ASMtuple
        over <- GenomicRanges::findOverlaps(tupgr, res_dame)
        
        if (window != 0) {
            win <- c(seq(from = (queryHits(over)[1] - window), 
                to = (queryHits(over)[1] - 1), by = 1), queryHits(over), 
                seq(from = (utils::tail(queryHits(over), n = 1) + 
                1), to = (utils::tail(queryHits(over), n = 1) + 
                window), by = 1))
        } else {
            win <- queryHits(over)
        }
        
        subASMtuple <- as.data.frame(ASMtuple[win, ])
        submeth <- as.data.frame(meth[win, ])
        subASMtuple$pos <- submeth$pos <- tupgr$midpt[win]
        subASMtuple_long <- reshape2::melt(subASMtuple, id.vars = "pos", 
            measure.vars = colnames(ASMtuple))
        subASMtuple_long$score <- "ASMtuple"
        
        # Marginal meth
        submeth_long <- reshape2::melt(submeth, id.vars = "pos", 
            measure.vars = colnames(meth))
        submeth_long$score <- "meth"
        
    }
    
    if ((!is.null(derASM)) & (!is.null(ASM))) {
        message("Using both scores")
        full_long <- rbind(subASMtuple_long, submeth_long, subASMsnp_long, 
            subref_long, subalt_long)
        full_long$score <- factor(full_long$score, levels = c("ASMtuple", 
            "meth", "ASMsnp", "REF:meth", "ALT:meth"))
        full_long$group <- colData(derASM)$group[match(full_long$variable, 
            colData(derASM)$samples)]
    }
    
    if (is.null(ASM) & !is.null(derASM)) {
        message("Using ASMsnp score")
        full_long <- rbind(subASMsnp_long, subref_long, subalt_long)
        full_long$score <- factor(full_long$score, levels = c("ASMsnp", 
            "REF:meth", "ALT:meth"))
        full_long$group <- colData(derASM)$group[match(full_long$variable, 
            colData(derASM)$samples)]
    }
    
    if (is.null(derASM) & !is.null(ASM)) {
        message("Using ASMtuple score")
        full_long <- rbind(subASMtuple_long, submeth_long)
        full_long$group <- colData(ASM)$group[match(full_long$variable, 
            colData(ASM)$samples)]
    }
    
    # To draw rectangle on region
    forect <- data.frame(xmin = start(dame), xmax = end(dame), 
        ymin = 0, ymax = Inf)
    
    # plot scores
    m1 <- ggplot(data = full_long) +
      geom_line(mapping = aes(x = .data$pos,
                              y = .data$value,
                              group = .data$variable,
                              color = .data$group), alpha = 0.5) + 
      geom_point(mapping = aes(x = .data$pos,
                                 y = .data$value,
                                 group = .data$variable, 
                                 color = .data$group)) +
      geom_rect(data = forect, aes(xmin = .data$xmin, 
                                   xmax = .data$xmax,
                                   ymin = .data$ymin,
                                   ymax = .data$ymax), alpha = 0.1) + 
      facet_grid(score ~ ., scales = "free_y") +
      theme_bw() + 
      xlab("position") +
      ggtitle("Scores")
    
    cord <- ggplot_build(m1)$layout$panel_scales_x[[1]]$range$range
    
    # plot SNP table
    if (plotSNP) {
        m2 <- ggplot(data = subSNP_long) + 
            geom_point(aes(x = .data$pos,
                           y = 1,
                           group = .data$variable, 
                           color = .data$snp.pos),
                       shape = 8, size = 1, 
                       fill = "white", stroke = 1) +
            facet_grid(variable ~ .) + 
            theme_bw() + 
            theme(panel.spacing = unit(0, "lines"), 
                axis.text.y = element_blank(), axis.ticks.y = element_blank(), 
                panel.grid.major = element_blank(), 
                panel.grid.minor = element_blank()) + 
            coord_cartesian(xlim = cord) + 
            ylab("") + 
            ggtitle("SNPs - for ASMsnp") + 
            xlab("position")
        
        if (!is.null(colvec)) {
            m2 <- m2 + scale_color_manual(values = colvec)
            m1 <- m1 + scale_color_manual(values = colvec)
        }
        
        p <- cowplot::plot_grid(m1, m2, ncol = 1, nrow = 2, rel_heights = c(3, 
            1), align = "v")
    } else {
        
        if (!is.null(colvec)) {
            p <- m1 + scale_color_manual(values = colvec)
        } else {
            p <- m1
        }
    }
    
    return(p)
}

#' Find DAMEs
#'
#' This function finds Differential Allele-specific MEthylated regions (DAMEs).
#' It uses the \code{\link{regionFinder}} function from \code{bumphunter}, and
#' asigns p-values either empirically or using the Simes method.
#'
#' The simes method has higher power to detect DAMEs, but the consistency in
#' signal across a region is better controlled with the empirical method, since
#' it uses \code{regionFinder} and \code{getSegments} to find regions with
#' t-statistics above a cuttof (controled with parameter \code{Q}), whereas
#' with the 'simes' option, we initially detects clusters of CpG sites/tuples,
#' and then test if at least 1 differential site/tuple is present in the
#' cluster.
#'
#' We recommend trying out different \code{maxGap} and \code{Q} parameters,
#' since the size and the effect-size of obtained DAMEs change with these
#' parameters.
#'
#' @param sa A \code{SummarizedExperiment} containing ASM values where each row
#'   correspond to a tuple/site and a column to sample/replicate.
#' @param design A design matrix created with \code{\link{model.matrix}}.
#' @param coef Column in \code{design} specifying the parameter to estimate.
#'   Default = 2.
#' @param contrast a contrast matrix, generated with
#'   \code{\link{makeContrasts}}.
#' @param Q The percentile set to get a cutoff value K. K is the value on the
#'   Qth quantile of the absolute values of the given (smoothed) t-statistics.
#'   Only necessary if \code{pvalAssign} = 'empirical'. Default = 0.5.
#' @param pvalAssign Choose method to assign pvalues, either 'simes' (default)
#'   or 'empirical'. This second one performs \code{maxPerms} number of
#'   permutations to calculate null statistics, and runs \code{regionFinder}.
#' @param maxGap Maximum gap between CpGs in a cluster (in bp). NOTE: Regions
#' can be as small as 1 bp. Default = 20.
#' @param smooth Whether smoothing should be applied to the t-Statistics.
#'   Default = TRUE.
#' @param maxPerms Maximum possible permutations generated. Only necessary if
#' \code{pvalAssign} = 'empirical'. Default = 10.
#' @param method The method to be used in limma's \code{\link{lmFit}}. The
#'   default is set to 'ls' but can also be set to 'robust', which is
#'   recommended on a real data set.
#' @param trend Passed to \code{\link{eBayes}}. Should an intensity-trend be
#' allowed for the prior variance? Default is that the prior variance is
#' constant, e.g. FALSE.
#' @param verbose If the function should be verbose. Default = TRUE.
#' @param ... Arguments passed to \code{\link{get_tstats}}.
#'
#' @return A data frame of detected DAMEs ordered by the p-value. Each row
#'   is a DAME and the following information is provided in the columns
#'   (some column names change depending on the \code{pvalAssign} choice):
#'
#'   - chr: on which chromosome the DAME is found.
#'   - start: The start position of the DAME.
#'   - end: The end position of the DAME.
#'   - pvalSimes: p-value calculated with the Simes method.
#'   - pvalEmp: Empirical p-value obtained from permuting covariate of interest.
#'   - sumTstat: Sum of t-stats per segment/cluster.
#'   - meanTstat: Mean of t-stats per segment/cluster.
#'   - segmentL: Size of segmented cluster (from \code{\link{getSegments}}).
#'   - clusterL: Size of original cluster (from \code{\link{clusterMaker}}).
#'   - FDR: Adjusted p-value using the method of Benjamini, Hochberg. (from
#'   \code{\link{p.adjust}}).
#'   - numup: Number of sites with ASM increase in cluster (only for Simes).
#'   - numdown: Number of sites with ASM decrease in cluster (only for Simes).
#'
#' @md
#'
#' @examples
#' data(readtuples_output)
#' ASM <- calc_asm(readtuples_output)
#' grp <- factor(c(rep('CRC',3),rep('NORM',2)), levels = c('NORM', 'CRC'))
#' mod <- model.matrix(~grp)
#' dames <- find_dames(ASM, mod, verbose = FALSE)
#'
#' @export
#'
#'
find_dames <- function(sa, design, coef = 2, contrast = NULL, 
    smooth = TRUE, Q = 0.5, pvalAssign = "simes", maxGap = 20, 
    verbose = TRUE, maxPerms = 10, method = "ls", trend = FALSE, 
    ...) {
    
    
    if ("asm" %in% names(assays(sa))) 
        message("Using ASMtuple", appendLF = TRUE)
    if ("der.ASM" %in% names(assays(sa))) 
        message("Using ASMsnp", appendLF = TRUE)
    
    pre_sa <- sa
    
    # get tstats with limma
    sat <- get_tstats(pre_sa, design, maxGap = maxGap, coef = coef, 
        contrast = contrast, smooth = smooth, method = method, 
        trend = trend, ...)
    
    # choose smoothed if true
    if (smooth) {
        sm_tstat <- S4Vectors::mcols(sat)$smooth_tstat
    } else {
        sm_tstat <- S4Vectors::mcols(sat)$tstat
    }
    
    # choose position to find regions
    if (names(assays(sa))[1] == "asm") {
        midpt <- S4Vectors::mcols(sat)$midpt
    } else {
        midpt <- BiocGenerics::start(sat)
    }
    
    message("Detecting DAMEs", appendLF = TRUE)
    # detect dames
    K <- stats::quantile(abs(sm_tstat), Q, na.rm = TRUE)
    
    if (pvalAssign == "simes") {
        
        rf <- simes_pval(sat, sm_tstat, midpt)
        rf <- rf[order(rf$pvalSimes), ]
        rf$FDR <- stats::p.adjust(rf$pvalSimes, method = "BH")
        
    } else if (pvalAssign == "empirical") {
        
        rf <- bumphunter::regionFinder(x = sm_tstat, 
            chr = as.character(Seqinfo::seqnames(sat)), 
            pos = midpt, cluster = S4Vectors::mcols(sat)$cluster, 
            cutoff = K, maxGap = maxGap, assumeSorted = TRUE, 
            order = FALSE, verbose = verbose)
        
        rf$pvalEmp <- empirical_pval(presa = pre_sa, design = design, 
            rforiginal = rf, coeff = coef, cont = contrast, smooth = smooth, 
            maxPerms = maxPerms, Q = Q, maxGap = maxGap, method = method, 
            ...)
        
        rf <- rf[order(rf$pvalEmp), ]
        
        rf$FDR <- stats::p.adjust(rf$pvalEmp, method = "BH")
        
        rf <- rf[, -c(6:8)]
        colnames(rf) <- c("chr", "start", "end", "meanTstat", 
            "sumTstat", "segmentL", "clusterL", "pvalEmp", "FDR")
        
    }
    if (verbose) 
        message(nrow(rf), " DAMEs found.", appendLF = TRUE)
    return(rf)
}

#' Read in list of methtuple files
#'
#' This function reads in a list of files obtained from the methtuple tool. It
#' filters out tuples based on the set minimum coverage (min_cov) and the
#' maximum allowed distance (maxGap) between two genomic positions in a tuple.
#'
#'
#' @param files List of methtuple files.
#' @param sampleNames Names of files in the list.
#' @param minCoverage The minimum coverage per tuple. Tuples with a coverage <
#'   minCoverage are filtered out. Default = 2.
#' @param verbose If the function should be verbose.
#' @param maxGap The maximum allowed distance between two positions in a tuple.
#'   Only distances that are <= maxGap are kept. Default = 150 base pairs.
#'
#' @return A list of data frames, where each data frame corresponds to one file.
#' @examples
#' DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
#' get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
#'
#' tuple_files <- list.files(DATA_PATH_DIR, '.tsv.gz')
#' tuple_files <- get_data_path(tuple_files)
#' ASM <- read_tuples(tuple_files, c('CRC1', 'NORM1'))
#' 
#' @export
#'
#' 
read_tuples <- function(files, sampleNames, minCoverage = 2, 
    maxGap = 20, verbose = TRUE) {
    pos2 <- pos1 <- NULL
    MM <- UU <- MU <- UM <- NULL
    
    # read in methtuple files
    methtuple_cols <- readr::cols(chr = readr::col_character(), 
        strand = readr::col_character(), pos1 = readr::col_integer(), 
        pos2 = readr::col_integer(), MM = readr::col_integer(), 
        MU = readr::col_integer(), UM = readr::col_integer(), 
        UU = readr::col_integer())
    
    names(files) <- sampleNames
    
    methtuple_list <- lapply(files, function(u) {
        if (verbose) 
            message("Reading ", u)
        readr::read_tsv(u, col_types = methtuple_cols, progress = FALSE)
    })
    
    # filter out low coverage and maxGap
    if (verbose) 
        message("Filtering and sorting: ", appendLF = FALSE)
    methtuple_list <- lapply(methtuple_list, function(df) {
        if (verbose) 
            message(".", appendLF = FALSE)
        df$cov <- with(df, MM + UU + UM + MU)
        df$inter_dist <- with(df, pos2 - pos1)
        w <- df$cov >= minCoverage & df$inter_dist <= maxGap
        df <- df[w, ]
        df[order(df$chr, df$pos1, df$pos2), ]
    })
    if (verbose) 
        message(" done.")
    
    methtuple_list
}

#' Multidimensional scaling plot of distances between methylation 
#' proportions (beta values)
#'
#' Same as \code{\link{plotMDS}}, except for an arc-sine transformation of the
#' methylation proportions.
#'
#' @param x \code{RangedSummarizedExperiment}, output from
#'   \code{calc_derivedasm} or \code{calc_asm}.
#' @param group Vector of group or any other labels, same length as number of
#'   samples.
#' @param top Number of top CpG sites used to calculate pairwise distances.
#' @param coverage Minimum number of reads covering a CpG site on each allele.
#'   Default = 5.
#' @param adj Text adjustment in y-axis. Default = 0.2.
#' @param pointSize Default = 4.
#'
#' @return Two-dimensional MDS plot.
#'
#' @importFrom SummarizedExperiment assays
#' @import ggplot2
#' 
#' @examples 
#' data(readtuples_output)
#' ASM <- calc_asm(readtuples_output)
#' grp <- factor(c(rep('CRC',3),rep('NORM',2)), levels = c('NORM', 'CRC'))
#' methyl_MDS_plot(ASM, grp)
#'
#' @export
methyl_MDS_plot <- function(x, group, top = 1000, coverage = 5, 
    adj = 0.02, pointSize = 4) {
    
    
    if (names(assays(x))[1] == "asm") {
        
        asm <- assays(x)[["asm"]]
        asm.red <- asm[rowSums(!is.na(assays(x)[["cov"]]) & 
            assays(x)[["cov"]] >= 
            coverage) == BiocGenerics::ncol(x), ]
        
        #mds_meth <- limma::plotMDS(asm.red, top = top, 
        #                            plot = FALSE)$cmdscale.out
        mds_meth <- limma::plotMDS(asm.red, top = top, 
                                    plot = FALSE)$eigen.vectors
        
    } else {
        
        full.cov <- assays(x)[["ref.cov"]] + assays(x)[["alt.cov"]]
        filt <- rowSums(full.cov >= coverage & 
            !is.na(full.cov)) == BiocGenerics::ncol(x)
        xfilt <- x[filt, ]
        
        # transform derived ASM
        methsTR <- as.matrix(assays(xfilt)[["der.ASM"]])
        
        methsTR <- asin(2 * methsTR - 1)
        bad <- rowSums(is.finite(methsTR)) < ncol(methsTR)
        if (any(bad)) 
            methsTR <- methsTR[!bad, , drop = FALSE]
        
        mds_meth <- limma::plotMDS(methsTR, top = top, 
                                    plot = FALSE)$eigen.vectors
    }
    
    df <- data.frame(dim1 = mds_meth[, 1], dim2 = mds_meth[, 
        2], names = colnames(x), treat = group)
    
    ggplot() +
      geom_point(data = df,
                 mapping = aes(x = .data$dim1, 
                               y = .data$dim2,
                               color = .data$treat),
                 size = pointSize) +
      geom_text(data = df, 
                mapping = aes(x = .data$dim1,
                              y = .data$dim2 - adj,
                              label = .data$names)) + 
      theme_bw()
    
}

#' extract_bams() output.
#'
#' 4 Patients from a previous study (Parker et al, 2018.) with colorectal cancer
#' were sequenced and the normal and cancerous tissue of each patient was
#' obtained. The data includes a subset of chromosome 19.
#' 
#' @format A large list with 8 elements. Each element is a list of
#'   \code{GRanges} for each sample. Each GRanges in the list includes the
#'   location of the CpG sites contained in the reads for each SNP. The GRanges
#'   metadata table contains: 
#'   \describe{
#' \item{\code{cov.ref}}{Number of reads of "reference" allele in that SNP}
#' \item{\code{cov.alt}}{Number of reads of "alternative" allele in that SNP}
#' \item{\code{meth.ref}}{Number of methylated reads of "reference" allele in
#' that SNP}
#' \item{\code{cov.ref}}{Number of methylated reads of "alternative" allele in
#' that SNP}
#' \item{\code{snp}}{The SNP containing the reads}
#' }
#' 
#' For further details, 
#' see \url{https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6949/}
#' sample names in in ArrayExpress do not necessarily match names given here!
"extractbams_output"

#' read_tuples() output.
#'
#' 3 Patients from a previous study (Parker et al, 2018.) with colorectal cancer
#' were sequenced and the normal and cancerous tissue of each patient was
#' obtained. The data includes a subset of chromosome 19. Here one normal sample
#' is not included.
#' 
#' @format A large list with 5 elements. Each element is a \code{tibble} with
#'   the coordinates of the pairs of CpG sites (tuples). Rest of the tibble
#'   contains:
#'   \describe{
#' \item{\code{MM}}{Number of reads with both CpG sites methylated}
#' \item{\code{MU}}{Number of reads with first CpG site methylated}
#' \item{\code{UM}}{Number of reads with second CpG site methylated}
#' \item{\code{UU}}{Number of reads with both CpG sites unmethylated}
#' \item{\code{cov}}{Coverage, total reads at tuple}
#' \item{\code{inter_dist}}{Distance in bp between CpG sites}
#' }
#' 
#' For further details, 
#' see \url{https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6949/}
#' sample names in in ArrayExpress do not necessarily match names given here!
"readtuples_output"

#' Calculate empirical region-level p-value
#'
#' This function permutes the coefficient of interest and re-runs
#' \code{\link{get_tstats}} and \code{\link{regionFinder}} for each permutation.
#' Code for permutations copied from the \code{dmrseq} function from the package
#' of the same name.
#'
#' @param presa SExperiment output from \code{calc_derivedasm} or
#'   \code{calc_asm}.
#' @param design design matrix.
#' @param rforiginal data.frame of DAMEs calculated with original design.
#' @param coeff Coefficient of interest to permute.
#' @param cont same as in \code{get_tstats}.
#' @param smooth Boolean.
#' @param maxPerms Maximum possible permutations generated. Default = 10.
#' @param Q Quantile for cuttof.
#' @param maxGap Same as other functions in the package.
#' @param method lmFit method.
#' @param ... Passed to \code{get_tstats} and then to \code{loessByCluster}.
#' @return Vector of empirical p-values.
#' @keywords internal
#' 
empirical_pval <- function(presa, design, rforiginal, coeff, 
    cont, smooth, maxPerms = 10, Q, maxGap, method, ...) {
    
    sampleSize <- table(design[, coeff])
    
    # Since we allow the user to choose the coeff from the
    # design.matrix, I will never have more than 2 coefs, as
    # dmrseq does
    
    # limit possible number of perms!
    if (length(unique(design[, coeff])) == 2 && choose(nrow(design), 
        min(sampleSize)) < 5e+05) {
        
        perms <- utils::combn(seq(1, nrow(design)), min(sampleSize))
        # Remove redundant permutations (if balanced)
        if (length(unique(table(design[, coeff]))) == 1) {
            perms <- perms[, seq_len(ncol(perms)/2)]
        }
        
        # restrict to unique permutations that don't include any
        # groups consisting of all identical conditions
        rmv <- NULL
        for (p in seq_len(ncol(perms))) {
            if (length(unique(design[perms[, p], coeff])) == 
                1) {
                rmv <- c(rmv, p)
            }
        }
        
        if (length(rmv) > 0) 
            perms <- perms[, -rmv]
        
        # subsample permutations based on similarity to original
        # partition gives preference to those with the least
        # similarity
        if (maxPerms < ncol(perms)) {
            similarity <- apply(perms, 2, function(x) {
                max(table(design[x, coeff]))
            })
            perms.all <- perms
            perms <- NULL
            levs <- sort(unique(similarity))
            l <- 1
            num <- 0
            while (!(num == maxPerms) && l <= length(levs)) {
                keep <- sample(which(similarity == levs[l]), 
                min(maxPerms - num, sum(similarity == levs[l])))
                perms <- cbind(perms, perms.all[, keep])
                l <- l + 1
                num <- ncol(perms)
            }
        }
    } else message("Too many samples!")
    
    # Detect permuted dames
    message("Generating ", ncol(perms), " permutations", appendLF = TRUE)
    
    areas <- apply(perms, 2, function(i) {
        
        # message('Permutation ', num_perms[i], appendLF = TRUE)
        reorder <- i
        designr <- design
        
        if (length(unique(design[, coeff])) == 2 && !nrow(perms) == 
            nrow(designr)) {
            designr[, coeff] <- 0
            designr[reorder, coeff] <- 1
        } else {
            designr[, coeff] <- designr[reorder, coeff]
        }
        
        sa_perm <- get_tstats(presa, design = designr, coef = coeff, 
            contrast = cont, maxGap = maxGap, smooth = smooth, 
            verbose = TRUE, method = method, ...)
        
        # choose smoothed if true
        if (smooth) {
            sm_tstat <- S4Vectors::mcols(sa_perm)$smooth_tstat
        } else {
            sm_tstat <- S4Vectors::mcols(sa_perm)$tstat
        }
        
        # choose position to find regions
        if (names(assays(sa_perm))[1] == "asm") {
            midpt <- S4Vectors::mcols(sa_perm)$midpt
        } else {
            midpt <- BiocGenerics::start(sa_perm)
        }
        
        K <- stats::quantile(abs(sm_tstat), Q, na.rm = TRUE)
        permrf <- bumphunter::regionFinder(x = sm_tstat, 
            chr = as.character(Seqinfo::seqnames(sa_perm)), 
            pos = midpt, cluster = S4Vectors::mcols(sa_perm)$cluster, 
            cutoff = K, maxGap = maxGap, verbose = FALSE)
        return(abs(permrf$area))
    })
    
    # if( put a condition to check if areas actually has
    # something
    all_areas <- sort(unlist(areas))
    total_areas <- length(all_areas)
    
    pvalEmp <- vapply(rforiginal$area, function(a) {
        pperm <- (sum(all_areas > abs(a)) + 1)/(total_areas + 
            1)
        return(pperm)
    }, FUN.VALUE = double(1))
    
    return(pvalEmp)
}

#' Calculate region-level p-value
#'
#' This function uses the Simes method to calculate a regional-level p-value
#' based on the single-eBayes p-values. It highly depends on the choice of
#' \code{maxGap} in \code{find_dames}.
#'
#' When used as a FDR-control method, for positively correlated P-values, Simes
#' method is even closer to the nominal alpha level than the Bonferroni-Holm
#' method.
#'
#' @param sat Output from \code{\link{get_tstats}}.
#' @param smtstat (Smoothed) tstat vector from \code{\link{get_tstats}}.
#' @param midpt Coordinate vector for each CpG site/tuple.
#' @return Vector of summarized pvals
#' @keywords internal
#' 
simes_pval <- function(sat, smtstat, midpt) {
    
    # try to use this strategy using full clusters, not segments
    # which means choosing the maxGap for cluster in get_tstat is
    # the most important parameter for pval method
    
    pval <- S4Vectors::mcols(sat)$p.value
    cluster <- mcols(sat)$cluster
    cluster.ids <- unique(cluster)
    
    chr <- as.character(seqnames(sat))
    starts <- midpt
    ends <- midpt
    
    simes <- function(pval) min((length(pval) * pval[order(pval)])/seq(from = 1,
        to = length(pval), by = 1))
    
    realregs <- data.frame(
        chr = vapply(cluster.ids, function(Index) chr[cluster == Index][1], 
            FUN.VALUE = character(1)), 
        start = vapply(cluster.ids, function(Index) 
            min(starts[cluster == Index]), FUN.VALUE = double(1)), 
        end = vapply(cluster.ids, function(Index) max(ends[cluster == Index]), 
            FUN.VALUE = double(1)), 
        meanTstat = vapply(cluster.ids, function(Index) 
            mean(smtstat[cluster == Index]), FUN.VALUE = double(1)), 
        sumTstat = vapply(cluster.ids, function(Index) 
            sum(smtstat[cluster == Index]), FUN.VALUE = double(1)), 
        pvalSimes = vapply(cluster.ids, function(Index) simes(pval[cluster == 
            Index]), FUN.VALUE = double(1)), 
        clusterL = vapply(cluster.ids, function(Index) 
            length(cluster[cluster == Index]), FUN.VALUE = double(1)), 
        numup = vapply(cluster.ids, function(Index) 
            sum(smtstat[cluster == Index] > 0), FUN.VALUE = double(1)), 
        numdown = vapply(cluster.ids, function(Index) 
            sum(smtstat[cluster == Index] < 0), FUN.VALUE = double(1)))
    
    return(realregs)
}

#' Get t-Statistics
#'
#' This function calculates a moderated t-Statistic per site or tuple using
#' \code{limma}'s \code{\link{lmFit}} and \code{\link{eBayes}} functions. It
#' then smoothes the obtained t-Statistics using \code{bumphunter}'s
#' \code{\link{smoother}} function.
#'
#' The smoothing is done on genomic clusters consisting of CpGs that are close
#' to each other. In the case of tuples, the midpoint of the two genomic
#' positions in each tuple is used as the genomic position of that tuple, to
#' perform the smoothing.The function takes a \code{RangedSummarizedExperiment}
#' generated by \code{\link{calc_derivedasm}} or \code{\link{calc_asm}}
#' containing ASM across samples, and the index of control and treatment
#' samples.
#'
#' @param sa A SummarizedExperiment containing ASM values where each row and
#'   column correspond to a tuple/site and sample respectively.
#' @param design a design matrix created with \code{\link{model.matrix}}.
#' @param coef Column in model.matrix specifying the parameter to estimate.
#' Default = 2. If \code{contrast} specified, column with contrast of interest.
#' @param contrast a contrast matrix, generated with
#'   \code{\link{makeContrasts}}.
#' @param method The method to be used in limma's \code{\link{lmFit}}. The
#'   default is set to 'ls' but can also be set to 'robust', which is
#'   recommended on a real data set.
#' @param trend Passed to \code{\link{eBayes}}. Should an intensity-trend be
#' allowed for the prior variance? Default is that the prior variance is
#' constant, e.g. FALSE.
#' @param smooth Whether smoothing should be applied to the t-Statistics.
#'   Default = FALSE. If TRUE, wherever smoothing is not possible, the
#'   un-smoothed t-stat is used instead.
#' @param maxGap The maximum allowed gap between genomic positions for
#'   clustering of genomic regions to be used in smoothing. Default = 20.
#' @param verbose Set verbose. Default = TRUE.
#' @param filter Remove empty tstats. Default = TRUE.
#' @param ... Arguments passed to \code{\link{loessByCluster}}. Only used if
#'   \code{smooth} = TRUE.
#'
#' @return A vector of t-Statistics within the
#'   \code{RangedSummarizedExperiment}.
#' @importFrom BiocGenerics start
#' @importFrom SummarizedExperiment assays
#' 
#' @examples 
#' data(readtuples_output)
#' ASM <- calc_asm(readtuples_output)
#' grp <- factor(c(rep('CRC',3),rep('NORM',2)), levels = c('NORM', 'CRC'))
#' mod <- model.matrix(~grp)
#' tstats <- get_tstats(ASM, mod)
#'
#' @export

get_tstats <- function(sa, design, contrast = NULL, method = "ls", 
    trend = FALSE, smooth = FALSE, maxGap = 20, coef = 2, verbose = TRUE, 
    filter = TRUE, ...) {
    
    # choose SumExp type
    if (names(assays(sa))[1] == "asm") {
        asm <- assays(sa)[["asm"]]
    } else {
        asm <- assays(sa)[["der.ASM"]]
    }
    
    # moderated t-statistic using specified column in the design
    # matrix
    if (verbose) 
        message("Calculating moderated t-statistics", appendLF = TRUE)
    
    fit <- limma::lmFit(object = asm, design = design, method = method)
    if (is.matrix(contrast)) {
        fit.cont <- limma::contrasts.fit(fit, contrasts = contrast)
        fit2 <- limma::eBayes(fit.cont, trend = trend)
        
    } else {
        fit2 <- limma::eBayes(fit, trend = trend)
    }
    
    S4Vectors::mcols(sa)$tstat <- fit2$t[, coef]
    S4Vectors::mcols(sa)$p.value <- fit2$p.value[, coef]
    
    
    if (names(assays(sa))[1] == "asm") {
        midpt <- S4Vectors::mcols(sa)$midpt
    } else {
        midpt <- start(sa)
    }
    
    S4Vectors::mcols(sa)$cluster <- bumphunter::clusterMaker(
        chr = as.character(Seqinfo::seqnames(sa)), 
        pos = midpt, maxGap = maxGap, assumeSorted = TRUE)
    
    # smooth moderated t-stats
    if (smooth) {
        
        if (verbose) 
            message("Smoothing moderated t-statistics", appendLF = TRUE)
        
        smooth <- bumphunter::loessByCluster(y = S4Vectors::mcols(sa)$tstat, 
            x = midpt, cluster = S4Vectors::mcols(sa)$cluster, 
            verbose = verbose, ...)
        
        S4Vectors::mcols(sa)$smooth_tstat <- smooth$fitted[, 
            1]
        
        # replace empty smoothed value for original
        S4Vectors::mcols(sa)$smooth_tstat <- ifelse(
            is.na(S4Vectors::mcols(sa)$smooth_tstat), 
            S4Vectors::mcols(sa)$tstat, S4Vectors::mcols(sa)$smooth_tstat)
    }
    
    # filter for empty tstats
    if (filter) 
        sa <- sa[!is.na(S4Vectors::mcols(sa)$tstat), ]
    
    return(sa)
}

#' Divide read names by allele
#'
#' Takes a GenomicAlignments object and returns a list of read names dividied by
#' allele.
#'
#' @param alns GenomicAlignments object.
#' @param snp GRanges object containing SNP location.
#' @param v Nucleotide of reference (or alternative) allele.
#'
#' @return A named list of vectors, each vector containing read names for each
#'   allele.
#' @importFrom BiocGenerics start
#' @importFrom BiocGenerics end
#' @importFrom S4Vectors mcols
#' @importFrom stringr str_extract_all
#' @importFrom stringr str_extract
#' @importFrom stringr str_locate_all
#' @keywords internal
splitReads <- function(alns, v, snp) {
    
    fullMD <- mcols(alns)$MD
    fullstart <- start(alns)
    snp.loc <- start(snp)
    
    # Use MD tag from bam to check which reads have the ref or
    # alt allele
    alleles <- mapply(function(a, x, snp.loc) {
        
        # extract matches and mismatches from MD tag
        tag <- getMD(a)
        MDtag <- tag$MDtag
        nucl.num <- tag$nucl.num
        
        # Get right location of snp in sequence of read
        snp.start <- snp.loc - x + 1
        
        # Get the snp (location) from the specific read
        count <- 0
        
        for (i in seq_along(nucl.num)) {
            count <- count + nucl.num[i]
            if (count >= snp.start) {
                break
            }
        }
        return(MDtag[i])
    }, a = fullMD, x = fullstart, MoreArgs = list(snp.loc = snp.loc), 
        USE.NAMES = FALSE)
    
    # Get read names from alt and ref reads
    names(alleles) <- names(alns)
    ref.reads <- sort(names(which(alleles != v)))
    alt.reads <- sort(names(which(alleles == v)))
    
    return(list(ref.reads = unique(ref.reads), alt.reads = unique(alt.reads)))
}


#' MDtag parser
#'
#' Takes a GenomicAlignments object containing the MDtag, and transforms it
#' into a vector of characters and numbers
#'
#' @param a Vector of MDtags (single characters)
#'
#' @return A named list of vectors, each vector a parsed version of MDtag:
#' - nucl.num: Numeric representation of MDtag.
#' - MDtag: a split version of MDtag
#' @keywords internal
getMD <- function(a) {
    
    # extract matches and mismatches from MD tag
    numbers <- str_extract_all(a, "[0-9]{1,}")[[1]]
    numbers.loc <- str_locate_all(a, "[0-9]{1,}")[[1]][, 1]
    
    # nucl <- str_extract_all(a, '[A-Z^]{1,}')[[1]]
    nucl <- str_extract_all(a, "[A-Z^]{1}")[[1]]
    nucl.loc <- str_locate_all(a, "[A-Z^]{1}")[[1]][, 1]
    
    MDtag <- character(nchar(a))
    MDtag[numbers.loc] <- numbers
    MDtag[nucl.loc] <- nucl
    
    # remove empty and ^ positions
    MDtag <- MDtag[(MDtag != "" & MDtag != "^")]
    
    # extract MDtag as numbers
    if (length(MDtag) == 1) {
        nucl.num <- as.integer(MDtag)
    } else {
        nucl.num <- numeric(length(MDtag))
        nucl.num[MDtag %in% c("A", "C", "G", "T", "N")] <- 1
        
        if (sum(nucl.num != 1) == length(numbers)) {
            nucl.num[nucl.num != 1] <- as.integer(numbers)
        } else {
            message("I don't understand this error")
        }
    }
    return(list(MDtag = MDtag, nucl.num = nucl.num))
}

context("test-split_reads")
library(GenomicAlignments)
library(GenomicRanges)
library(Rsamtools)

DATA_PATH_DIR <- system.file("extdata", ".", package = "DAMEfinder")
get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
bam.file <- get_data_path("NORM1_chr19_trim.bam")

snp <- GRanges(19, IRanges(267039, width = 1))

params <- ScanBamParam(
  tag = c("MD","XM","XR","XG"),
  which = snp)

alns.pairs <- readGAlignmentPairs(bam.file,
                                  use.names = TRUE,
                                  param = params)

alns <- unlist(alns.pairs)

test_that("end to end split_reads", {
  split <- splitReads(alns, "C", snp)
  expect_is(split, "list")
  expect_length(split$ref.reads, 37)
  expect_length(split$alt.reads, 42)
})

test_that("end to end get_MD", {
  a <- "2C0C3C7C2C7C1C0C1C1C1C2C2C0C1C2C2C0C6C7C0C0C0C0C0C4C3C1C0C7C5C0C1C5C1C5C1C8"
  mdtag <- getMD(a)
  expect_is(mdtag, "list")
  expect_length(mdtag$MDtag, 75)
  expect_length(mdtag$nucl.num, 75)
})

test_that("large insertion in MD", {
  a <- "2C0C3C7C2C7C1C0C1C1^ACGT33C1C2C2C"
  mdtag <- getMD(a)
  expect_is(mdtag, "list")
  expect_length(mdtag$MDtag, 31)
  expect_length(mdtag$nucl.num, 31)
})

test_that("single insertion in MD", {
  a <- "4C5C0C2C0C4C0G2C0C0C0C19C4^C2C2C0C1C4C5C10C5C5C4"
  mdtag <- getMD(a)
  expect_is(mdtag, "list")
  expect_length(mdtag$MDtag, 45)
  expect_length(mdtag$nucl.num, 45)
})

test_that("several insertions in MD", {
  a <- "3G3G0G2G2G1G11G10G0G0G10^CA13^AA3G0G0G5G3G2G4G0G19G3"
  mdtag <- getMD(a)
  expect_is(mdtag, "list")
  expect_length(mdtag$MDtag, 45)
  expect_length(mdtag$nucl.num, 45)
})

---
title: "DAMEfinder Workflow"
author: 
- name: Stephany Orjuela
  affiliation:
    - &Oncobit Oncobit AG
    - &SIB SIB Swiss Institute of Bioinformatics, Switzerland
  email: sorjuelal@gmail.com
- name: Dania Machlab
  affiliation:
    - &FMI Friedrich Miescher Institute for Biomedical Research, Basel
    - *SIB 
- name: Mark Robinson
  affiliation:
    - &DMLS Department of Molecular Life Sciences, University of Zurich 
    - *SIB
date: "`r Sys.Date()`"
package: DAMEfinder
output: 
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{DAMEfinder Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
bibliography: papers.bib
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(cache = 0, fig.width = 6, 
                      fig.height = 7)
```

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

# Introduction

## What is allele-specific methylation?

The phenomenon occurs when there is an asymmetry in methylation between one
specific allele and the alternative allele [@hu2013]. The best studied example
of allele-specific methylation (ASM) is genomic imprinting. When a gene is
imprinted, one of the parental alleles is hyper-methylated compared to the other
allele, which leads to parent-allele-specific expression. This asymmetry is
conferred in the gametes or very early in embryogenesis, and will remain for the
lifetime of the individual [@kelsey2013]. ASM not related to
imprinting, exhibits parental-specific methylation,
but is not inherited from the germline [@hanna2017]. Another example of ASM is X
chromosome inactivation in females. DAMEfinder detects ASM for several
bisulfite-sequenced (BS-seq) samples in a cohort, and performs differential
detection for regions that exhibit loss or gain of ASM.


# Overview

We focus on any case of ASM in which there is an imbalance in the methylation
level between two alleles, regardless of the allele of origin.

DAMEfinder runs in two modes: **SNP-based** (exhaustive-mode) and
**tuple-based** (fast-mode), which converge when differential ASM is detected.

## Why **SNP-based**?
This is the exhaustive mode because it extracts an ASM score for every CpG site
in the reads containing the SNPs in a VCF file. Based on this score, DAMEs are
detected. From a biological point of view, you might want to run this mode if
you are interested in loss or gain of allele-specificity linked to somatic or
germline heterozygous SNPs (sequence-dependent ASM). More specifically, you
could detect genes that exhibit loss of imprinting (e.g. as in colorectal cancer
[@cui2002]).

## Why **tuple-based**?
To run the **tuple-based** mode you have to run
[methtuple](https://github.com/PeteHaitch/methtuple)[@hickey2015] first. The
methtuple output is the only thing needed for this mode. I call this the
fast-mode because you don't need SNP information. The assumption is that
intermediate levels of methylation represent ASM along the genome. For example,
we have shown (paper in prep) that the ASM score can distinguish females from
males in the X chromosome. Using SNP information this wouldn't be possible.

## Installation

```{r eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("DAMEfinder")
```

# Get bam files 
In order to run any of the two modes, you must obtain aligned bam files using
[`bismark`](http://www.bioinformatics.babraham.ac.uk/projects/bismark/). Here we
demonstrate how to generate these starting from paired-end fastq files of
bisulfite-treated reads:

```{bash eval=FALSE}
#Check quality of reads
fastqc -t 2  sample1_R1.fastq.gz sample1_R2.fastq.gz

#Trim reads to remove bad quality regions and adapter sequence
trim_galore --paired sample1_R1.fastq.gz sample2_R2.fastq.gz
```

To trim the reads we use [`Trim
Galore`](https://github.com/FelixKrueger/TrimGalore) and specify the use of
paired reads. By default it will remove any adapter sequence it recognizes.
Please refer to the user guide for further specifications.

```{bash eval=FALSE}
#Build bisulfite reference 
bismark_genome_preparation <path_to_genome_folder>

#run Bismark
bismark -B sample1 --genome <path_to_genome_folder> 
    -1 sample1_R1_val_1.fq.gz 
    -2 sample1_R2_val_2.fq.gz

#deduplicate (optional)
deduplicate_bismark -p --bam sample1_pe.bam

#sort and index files
samtools sort -m 20G -O bam -T _tmp 
    -o sample1_pe.dedupl_s.bam sample1_pe.deduplicated.bam
samtools index file1_pe.dedupl_s.bam
```

Before the alignment, you must download a reference fasta file from
[Ensembl](https://www.ensembl.org/info/data/ftp/index.html) or
[Gencode](https://www.gencodegenes.org/), and generate a bisulfite converted
reference. For this we use `bismark_genome_preparation` from the `bismark`
suite, and specify the folder that contains the fasta file with its index
file. Depending on the library type and kit used to obtain the reads, you may
want to deduplicate your bam files (e.g. TruSeq). Please refer to the [user
guide](http://www.bioinformatics.babraham.ac.uk/projects/bismark/)
for further explanation and specifications.


# SNP-based (aka slow-mode)

To run the SNP-based mode, you need to additionally have a VCF file including
the heterozygous SNPs per sample. If you do not have this, we recommend using
the tuple-based mode, or running
[`Bis-SNP`](http://people.csail.mit.edu/dnaase/bissnp2011/) to obtain variant
calls from bisulfite-converted reads.
<!-- include example to run Bis-SNP? -->

## Example Workflow 

In this example we use samples from two patients with colorectal cancer from a
published dataset [@parker2018]. For each patient two samples were taken:
`NORM#` corresponds to normal mucosa tissue and `CRC#` corresponds to the paired
adenoma lesion. Each of these samples was sequenced using targeted BS-seq
followed by variant calling using `Bis-SNP`.

### Obtain allele-based methylation calls

Similar to the `bismark_methylation_extractor`, we obtain methylation calls.
However since we are interested in allele-specific methylation, we only extract
methylation for CpG sites that fall within reads including a SNP. For every SNP
in the VCF file an independent methylation call is performed by using
`extract_bams`, which "extracts" reads from the bam file according to the
alleles, and generates a `list` of `GRangesList`s:

```{r}
suppressPackageStartupMessages({
  library(DAMEfinder)
  library(SummarizedExperiment)
  library(GenomicRanges)
  library(BSgenome.Hsapiens.UCSC.hg19)
  })

bam_files <- c(system.file("extdata", "NORM1_chr19_trim.bam", 
                           package = "DAMEfinder"),
               system.file("extdata", "CRC1_chr19_trim.bam", 
                           package = "DAMEfinder"))

vcf_files <- c(system.file("extdata", "NORM1.chr19.trim.vcf", 
                           package = "DAMEfinder"),
               system.file("extdata", "CRC1.chr19.trim.vcf", 
                           package = "DAMEfinder"))

sample_names <- c("NORM1", "CRC1")

#Use another reference file for demonstration, and fix the seqnames
genome <- BSgenome.Hsapiens.UCSC.hg19
seqnames(genome) <- gsub("chr","",seqnames(genome))
reference_file <- DNAStringSet(genome[[19]], use.names = TRUE)
names(reference_file) <- 19

#Extract reads and extract methylation according to allele
snp.list <- extract_bams(bam_files, vcf_files, sample_names, reference_file,
                       coverage = 2)

#CpG sites for first SNP in VCF file from sample NORM1
snp.list$NORM1[[1]]

#CpG sites for first SNP in VCF file from sample CRC1
snp.list$CRC1[[1]]

```

For demonstration, we include bam files from chromosome 19, and shortened VCF
files. Typically we would run the function on an entire bam and VCF file, which
would generate a large output.

The function also takes as input the reference file used to generate the
alignments. For demonstration we use chromosome 19 of the `GRCh37.91` reference
fasta file.

### Summarize methylation calls across samples

We use `calc_derivedasm()` to generate a `RangedSummarizedExperiment` from the
large list we generated above:

```{r}

derASM <- calc_derivedasm(snp.list)

derASM
assays(derASM)
```

Every row in the object is a single CpG site, and each column a sample. It 
contains 6 matrices in `assays`:

* `der.ASM`: A derived SNP-based ASM defined as $abs(\frac{X^{r}_M}{X^{r}} -
\frac{X^{a}_M}{X^{a}})$, where $X$ is the coverage in the reference $r$ or
alternative allele $a$, and $X_M$ the number of methylated reads in $r$ or $a$.
Basically, CpG sites with values of 1 or close to 1 have more
allele-specificity. ASM of 1 represents the perfect scenario in which none of
the reads belonging to one allele are methylated, and the reads of the other
allele are completely methylated.

* NEW `z.ASM`: SNP-based ASM defined as a Z score in a two-proportions test:
$abs(\frac{p^{r}-p^{a}} {p(1-p)(1/X^{r} + 1/X^{a})})$, where $p$ is
$\frac{X_M}{X}$ of either the reference, the alternative or both alleles. This
score is more sensitive to the coverage at each CpG site, and has no upper
limit.

* `snp.table`: Location of the SNP associated to the CpG site.

* `ref.cov`: Coverage of the "reference" allele.

* `alt.cov`: Covearage of the "alternative" allele.

* `ref.meth`: Methylated reads from the "reference" allele.

* `alt.meth`: Methylated reads from the "alternative" allele.

You can access these assays as:

```{r}
x <- assay(derASM, "der.ASM")
head(x)
```


### Find DAMEs

Now we detect regions that show differential ASM. The function `find_dames()`
performs several steps:

 1. Obtains a moderated t-statistic per CpG site using `lmFit()` and `eBayes()`
 from the `limma` package. The statistic reflects a measure of difference
 between the conditions being compared, in this case normal Vs cancer. The
 t-statistic is optionally smoothed (`smooth` parameter).
 
 After this, two methods can be chosen (`pvalAssign` parameter):
 
 * Simes method: 
    2. (Default) Clusters of CpG sites are determined by closeness (`maxGap`),
    and a p-value for each cluster is calculated using the simes method, similar
    to the package `csaw` from @lun2014. With this approach, the p-value
    represents evidence against the null hypothesis that no sites are
    differential in the cluster.
    
  * Bumphunting method:
    2. CpG sites with a t-statistic above and below a certain cutoff (set with
    `Q`), are grouped into segments (after being clustered). This is done with
    the `regionFinder()` function from `bumphunter` [@jaffe2012].
    3. For each of these segments, a p-value is calculated empirically by
    permuting the groups (covariate) of interest. Depending on the number of
    samples, this can take longer than the Simes method. However the number of
    permutations can be controlled with `maxPerms`.
 

Here we show an example with a pre-processed set of samples: 4 colorectal cancer
samples, and their paired normal mucosa:

```{r}

data(extractbams_output)

#The data loaded is an output from `split_bams()`, therefore we run 
#`calc_derivedasm` to get the SummarizedExperiment
derASM <- calc_derivedasm(extractbams_output, cores = 1, verbose = FALSE)

#We remove all CpG sites with any NA values, but not 0s
filt <- rowSums(!is.na(assay(derASM, "der.ASM"))) == 8 
derASM <- derASM[filt,]

#set the design matrix
grp <- factor(c(rep("CRC",4),rep("NORM",4)), levels = c("NORM", "CRC"))
mod <- model.matrix(~grp)
mod

#Run default
dames <- find_dames(derASM, mod)

head(dames)

#Run empirical method
dames <- find_dames(derASM, mod, pvalAssign = "empirical")

head(dames)

```

A significant p-value represent regions where samples belonging to
one group (in this case the cancer samples), gain or lose allele-specificity
compared to the other group (here the normal group).

# tuple-based (aka fast-mode)

Before running the tuple-based mode, you must obtain files from the `methtuple`
tool to input them in the `read_tuples` function.


## Run Methtuple on bam files

Methtuple requires the input `BAM` files of paired-end reads to be sorted by
query name. For more information on the options in `methtuple`, refer to the
user [guide](https://github.com/PeteHaitch/methtuple). For example the `--sc`
option combines strand information.

```{bash, eval=FALSE}

# Sort bam file by query name
samtools sort -n -@ 10 -m 20G -O bam -T _tmp 
    -o sample1_pe_sorted.bam sample1_pe.deduplicated.bam

# Run methtuple
methtuple --sc --gzip -m 2 sample1_pe_sorted.bam
```


## Example Workflow 

### Read methtuple files 

We use the same samples as above to run `methtuple` and obtain `.tsv.gz` files.
We read in these files using `read_tuples` and obtain a list of `tibble`s, each
one for every sample:

```{r}
tuple_files <- c(system.file("extdata", "NORM1_chr19.qs.CG.2.tsv.gz", 
                             package = "DAMEfinder"),
                 system.file("extdata", "CRC1_chr19.qs.CG.2.tsv.gz", 
                             package = "DAMEfinder"))

sample_names <- c("NORM1", "CRC1")

tuple_list <- read_tuples(tuple_files, sample_names)

head(tuple_list$NORM1)
```

Each row in the `tibble` displays a tuple. The chromosome name and strand are
shown followed by `pos1` and `pos2`, which refer to the genomic positions of the
first and second CpG in the tuple. The `MM`, `MU`, `UM`, and `UU` counts of the
tuple are displayed where `M` stands for methylated and `U` for unmethylated.
For example, `UM` shows the read counts for the instances where `pos1` is
unmethylated and `pos2` is methylated. The coverage and distance between the two
genomic positions in the tuple are shown under `cov` and `inter_dist`
respectively.


### Calculate ASM Score

The `calc_asm` function takes the output from `read_tuples()`, and as in the
SNP-based mode, generates a `RangedSummarizedExperiment` where each row is a
tuple and each column is a sample. The object contains 6 assays including the
`MM`, `MU`, `UM`, and `UU` counts, as well as the total coverage and the
tuple-based ASM score. This score is a measure of ASM calculated directly from
the reads without the need of SNP information. Because of this, it is a lot
quicker than the SNP-based ASM, and is useful for more explorative purposes.

Equations \@ref(eq:asmGeneral), \@ref(eq:asmWeight) and \@ref(eq:asmTheta) show
how the score is calculated. The log odds ratio in equation \@ref(eq:asmGeneral)
provides a higher score the more `MM` and `UU` counts the tuple has, whereas a
higher `UM` and `MU` would indicate "random" methylation. The weight further
adds allele-specificity where a rather balanced MM:UU increases the score.


\begin{equation}
    ASM^{(i)} = log{ \Big\{ \frac{X_{MM}^{(i)} \cdot X_{UU}^{(i)}}{X_{MU}^{(i)}
    \cdot X_{UM}^{(i)}} \Big\} \cdot w_i }
    (\#eq:asmGeneral)
\end{equation}

\begin{equation}
    w_i = P(0.5-\epsilon < \theta < 0.5+\epsilon~|~ X_{MM}^{(i)}, X_{UU}^{(i)},
    \beta_1, \beta_2)
    (\#eq:asmWeight)
\end{equation}

\begin{equation}
    \theta^{(i)} | X_{MM}^{(i)}, X_{UU}^{(i)},\beta_1, \beta_2 \sim
    Beta(\beta_1+X_{MM}^{(i)}, \beta_2+X_{UU}^{(i)})
    (\#eq:asmTheta)
\end{equation}


where $\theta^{(i)}$ represents the moderated proportion of MM to MM+UU alleles.
The weight, $w_i$ is set such that the observed split between MM and UU alleles
can depart somewhat from 50/50, while fully methylated or unmethylated tuples,
which represents evidence for absence of allele-specificity, are attenuated to
0.  The degree of allowed departure can be set according to $\epsilon$, the
deviation from 50/50 allowed and the level of moderation, $\beta_1$ and
$\beta_2$.

```{r}

ASM_mat <- calc_asm(tuple_list)
ASM_mat
```


### Find DAMEs

As above, the `RangedSummarizedExperiment` is used to detect differential ASM.
Here we show an example with a pre-processed set of samples: 3 colorectal cancer
samples, an 2 normal mucosa samples

```{r}
#load package data
data(readtuples_output)

#run calc_asm and filter object
ASMscore <- calc_asm(readtuples_output)
filt <- rowSums(!is.na(assay(ASMscore, "asm"))) == 5 #filt to avoid warnings
ASMscore <- ASMscore[filt,]

#make design matrix (or specify a contrast)
grp <- factor(c(rep("CRC",3),rep("NORM",2)), levels = c("NORM", "CRC"))
mod <- model.matrix(~grp)

#run default and increase maxGap to get longer, more sparse regions
dames <- find_dames(ASMscore, mod, maxGap = 300)

head(dames)

#run alternative mode
dames <- find_dames(ASMscore, mod,  maxGap = 300, pvalAssign = "empirical")

head(dames)

```


# Visualization

## DAME tracks

After detecting a set of DAMEs you want to look at them individually. We do this
with the function `dame_track`.

Depending on which object I used to obtain my DAMEs (tuple or SNP mode), I
choose which SummarizedExperiment to input in the field `ASM` (for tuple), or
`derASM` (for SNP). Either way, the SummarizedExperiment must have the columns
`group` and `samples` in the `colData` field:

```{r, dametrack}

#Here I will use the tuple-ASM SummExp
colData(ASMscore)$group <- grp
colData(ASMscore)$samples <- colnames(ASMscore)

#Set a DAME as a GRanges. I choose a one from the tables we obtained above
dame <- GRanges(19,IRanges(323736,324622))

dame_track(dame = dame,
           ASM = ASMscore)
```

Because we used the tuple-ASM object, we get by default two tracks: the ASM
score, and the marginal methylation (aka beta-value).

The shaded square delimits the DAME we defined to plot. We can look at the
flanking regions of the DAME by changing `window` or `positions`. With `window`
we specify the number of CpG positions we want to add to the plot up and
down-stream. With `positions` we specify the number of base pairs.

```{r, dt2}
dame_track(dame = dame,
           ASM = ASMscore,
           window = 2)

```

If we use the SNP-ASM as input we get different tracks:

```{r, dt3}

dame <- GRanges(19,IRanges(387966,387983))

grp <- factor(c(rep("CRC",4),rep("NORM",4)), levels = c("NORM", "CRC"))
colData(derASM)$group <- grp

dame_track(dame = dame,
           derASM = derASM)

```

Here we get three tracks: the SNP-ASM score, and the methylation levels for each
allele. Since the ASM score here depends on SNPs, we can see what SNPs are
involved in the ASM calculation at each CpG position:

```{r, dt4}
dame_track(dame = dame,
           derASM = derASM,
           plotSNP = TRUE)

```

We see that the SNP located at `chr19:388,065` was the one used to split the
allele methylation.

If you put both SummarizedExperiments with a single DAME, you would get all the
tracks:
```{r, dt5}
dame_track(dame = dame,
           derASM = derASM,
           ASM = ASMscore)

```

Notice that the first two tracks depend on the tuple-ASM, hence each point
represents the midpoint between a pair of CpG sites.

If you think plotting all the samples separately is difficult to see, you can
use the function `dame_track_mean` to summarize:
```{r, dt6}
dame_track_mean(dame = dame,
           derASM = derASM,
           ASM = ASMscore)

```

As you can see, this region is not a very good DAME.

## Methyl-circle plot

A typical way of visualizing ASM is to look at the reads overlapping a
particular SNP, and the methylation state of the CpG sites in those reads (black
circles for methylated and white for unmethylated, see @shoemaker2010 for
examples). Here we offer this option with the function `methyl_circle_plot()`.
As input it takes a `GRanges` with the SNP of interest, and the bam, VCF and
reference files as in the `extract_bams()` function.

```{r, fig1}
#put SNP in GRanges (you can find the SNP with the dame_track function)
snp <- GRanges(19, IRanges(267039, width = 1)) #always set the width if your 
#GRanges has 1 site

snp

bam.file <- system.file("extdata", "CRC1_chr19_trim.bam", 
                        package = "DAMEfinder")

vcf.file <- system.file("extdata", "CRC1.chr19.trim.vcf", 
                        package = "DAMEfinder")

methyl_circle_plot(snp = snp, vcfFile = vcf.file, bamFile = bam.file, 
                   refFile = reference_file)
```


You can reduce the number of reads included with the option `sampleReads`, which
performs a random sampling of the number of reads to be shows per allele. The
number of reads can be specified with `numReads`.

If you are interested in a specific CpG site within this plot, you can include
an extra `GRanges` with its location, and the triangle at the bottom will point
to it:

```{r, fig2}

cpgsite <- GRanges(19, IRanges(266998, width = 1))

methyl_circle_plot(snp = snp, vcfFile = vcf.file, bamFile = bam.file, 
                   refFile = reference_file, cpgsite = cpgsite)
```

If you are instead interested in reads overlapping a CpG site, you can use
`methyl_circle_plotCpG()`, which is useful if you run the tuple-mode:

```{r, fig3}

cpgsite <- GRanges(19, IRanges(266998, width = 1))

methyl_circle_plotCpG(cpgsite = cpgsite, bamFile = bam.file, 
                      refFile = reference_file)
```

You can also limit both the SNP plot and the CpG plot to a specific window of
interest (to zoom in or out), or if you want to look at the specific DAME
region:

```{r, fig4}

#a random region
dame <- GRanges(19, IRanges(266998,267100))

methyl_circle_plot(snp = snp, vcfFile = vcf.file, bamFile = bam.file, 
                   refFile = reference_file, dame = dame)

```


## MDS plot

To plot a multidimensional scaling plot (MDS), we provide a wrapper to
`plotMDS()` from `limma`, which adjusts the ASM score to calculate the euclidean
distances. The input is a SummarizedExperiment, and the vector of covariates
to color the points by:

```{r, fig5}

grp <- factor(c(rep("CRC",3),rep("NORM",2)), levels = c("NORM", "CRC"))
methyl_MDS_plot(ASMscore, group = grp)

```

# Session Info

```{r}
utils::sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methyl_circle_plot.R
\name{methyl_circle_plotCpG}
\alias{methyl_circle_plotCpG}
\title{Draw methylation circle plot without SNP}
\usage{
methyl_circle_plotCpG(
  cpgsite = cpgsite,
  bamFile = bamFile,
  pointSize = 3,
  refFile = refFile,
  dame = NULL,
  order = FALSE,
  sampleName = NULL,
  sampleReads = FALSE,
  numReads = 20
)
}
\arguments{
\item{cpgsite}{GRanges object containing a single CpG site location of
interest}

\item{bamFile}{bismark bam file path}

\item{pointSize}{Size of methylation circles. Default = 3.}

\item{refFile}{fasta reference file path}

\item{dame}{(optional) GRanges object containing a region to plot}

\item{order}{Whether reads should be sorted by methylation status. Default=
False.}

\item{sampleName}{Plot title.}

\item{sampleReads}{Whether a subset of reads should be plotted.
Default = FALSE.}

\item{numReads}{Number of reads to plot, if sampleReads is TRUE. Default = 20}
}
\value{
Plot
}
\description{
Draws CpG site methylation status as points, in reads containing a specific
CpG site. Generates one plot per bam file.
}
\examples{
DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
bam_files <- get_data_path('NORM1_chr19_trim.bam')
sample_names <- 'NORM1'
#reference_file
suppressPackageStartupMessages({library(BSgenome.Hsapiens.UCSC.hg19)})
genome <- BSgenome.Hsapiens.UCSC.hg19
seqnames(genome) <- gsub("chr","",seqnames(genome))
dna <- DNAStringSet(genome[[19]], use.names = TRUE)
names(dna) <- 19

cpg <- GenomicRanges::GRanges(19, IRanges::IRanges(292082, width = 1))
methyl_circle_plotCpG(cpgsite = cpg,
 bamFile = bam_files,
 refFile = dna)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_asm.R
\name{calc_logodds}
\alias{calc_logodds}
\title{Calculate the log odds ratio}
\usage{
calc_logodds(s, eps = 1)
}
\arguments{
\item{s}{A data frame that contains the MM,UU,UM, and MU counts for each CpG
tuple for a particular sample. It is the resulting object of the
\code{read_tuples}.}

\item{eps}{Count added to each of the MM,UU,UM and MU counts to avoid
dividing by zero for example. The default is set to 1.}
}
\value{
The same object is returned with an additional column for the log
  odds ratio.
}
\description{
This function calculates the log odds ratio for a CpG tuple:
\code{(MM*UU)/(UM*MU)}, where 'M' stands for methylated and 'U' for
unmethylated. 'MM' reflects the count for instances the CpG pair is
methylated at both positions. The higher the MM and UU counts for that CpG
pair, the higher the log odds ratio.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_asm.R
\name{calc_weight}
\alias{calc_weight}
\title{Calculate Weight for ASM Score}
\usage{
calc_weight(MM, UU, beta = 0.5, a = 0.2)
}
\arguments{
\item{MM}{The read counts for where pos1 and pos2 of the tuple were both
methylated.}

\item{UU}{The read counts for where pos1 and pos2 of the tuple were both
unmethylated.}

\item{beta}{parameter for the beta distribution. In B(alpha,beta), we set
alpha=beta=0.5 by default.}

\item{a}{parameter for how far from 0.5 we go as a measure of allele-specific
methylation. The weight is the probability that the MM:(MM+UU) ratio is
between 0.5-a and 0.5+a. The default is set to 0.2.}
}
\value{
A number that reflects allele-specificity given MM and UU counts for
a CpG pair. This is used as a weight that is multiplied by the log odds
ratio to give the final ASM score of that tuple.

#calc_weight(MM=50, UU=50)
#0.9999716

#calc_weight(MM=20, UU=60)
#0.1646916
}
\description{
This function calculates a weight which reflects MM to UU balance, where M
stands for methylated and U for unmethylated. Given the MM and UU counts for
a particular tuple, the weight is obtained using the \code{link{pbeta}}
function.
}
\details{
For a given tuple with MM and UU counts, the weight that reflects
allele-scpecificity is calculated as follows:
\itemize{
\item Prior:\deqn{p(\theta|\alpha,\beta) \sim Beta(\alpha,\beta),} where
\eqn{\theta = \frac{MM}{MM+UU}} and \eqn{\alpha = \beta = 0.5}.
\eqn{p(\theta|\alpha,\beta)} represents our prior belief which is that
tuples are either fully methylated or fully unmethylated, rather than
allele-specifically methylated which is a much rarer event.
\item Likelihood: \deqn{p(x|\alpha,\beta) \propto \theta^{MM}(1-\theta)^{UU},}
where x is our observation (the MM and UU counts).
\item Posterior:\deqn{p(\theta|x) \propto p(x|\theta)*p(\theta|\alpha,\beta)}
\deqn{p(\theta|x) \propto \theta^{MM-0.5}(1-\theta)^{UU-0.5},}
where \eqn{\alpha = \beta = 0.5}. This posterior also follows a beta
distribution \eqn{\sim Beta(\alpha'=MM+0.5, \beta'=UU+0.5)}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_asm.R
\name{calc_score}
\alias{calc_score}
\title{Calculate score}
\usage{
calc_score(df, beta = 0.5, a = 0.2)
}
\arguments{
\item{df}{data frame of a sample containing all information per tuple
(MM,UU,UM and MU counts, as well as the log odds ratio per tuple) needed
for the ASM score.}

\item{beta}{parameter for the \code{calc_weight} function. It's the alpha and
beta values for the Beta function.}

\item{a}{parameter for the calc_weight function. The weight will be the
probability that the MM/(MM+UU) ratio lies between 0.5-a and 0.5+a.}
}
\value{
The same object with an additional column for the ASM score.
}
\description{
This function calculates the ASM score for every tuple in a given sample. The
ASM score is a multiplication of the log odds ratio by a weight that reflects
the extent of allele-specific methylation. This weight is obtained with the
\code{\link{calc_weight}} function.
}
\details{
This function returns an allele-specific methylation (ASM) score for
  every given tuple in a sample. The ASM score is a product of the log odds
  ratio and a weight reflecting a measure of allele-specificity using the MM
  and UU counts.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DAMEfinder-package.R
\docType{package}
\name{DAMEfinder}
\alias{DAMEfinder}
\title{DAMEfinder: Method to detect allele-specific methylation (ASM), and
differential ASM from Bisulfite sequencing data in R.}
\description{
The package allows the user to extract an ASM score in two ways: either from
a \code{bismark} bam file(s) and VCF file(s), or from the output from
\code{methtuple}. Either way the final output is a list of regions with
diferential allele-specific methylated between groups of samples of interest.
The package also provides functions to visualize ASM at the read level or the
score level
}
\section{DAMEfinder functions}{
 \code{calc_asm} extracts ASM for pairs of CpG
  sites from a methtuple file, \code{calc_derivedasm} extracts ASM at each
  CpG site linked to a SNP from the VCF file. Both functions generate a
  \code{RangedSummarizedExperiment}, which is the input for the main function
  \code{find_dames}, that generates a \code{data.frame} with regions
  exhibiting differential ASM between a number of samples.
}

\author{
Stephany Orjuela \email{sorjuelal@gmail.com}

Dania Machlab

Mark D Robinson \email{mark.robinson@imls.uzh.ch}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split_reads.R
\name{getMD}
\alias{getMD}
\title{MDtag parser}
\usage{
getMD(a)
}
\arguments{
\item{a}{Vector of MDtags (single characters)}
}
\value{
A named list of vectors, each vector a parsed version of MDtag:
- nucl.num: Numeric representation of MDtag.
- MDtag: a split version of MDtag
}
\description{
Takes a GenomicAlignments object containing the MDtag, and transforms it
into a vector of characters and numbers
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{extractbams_output}
\alias{extractbams_output}
\title{extract_bams() output.}
\format{
A large list with 8 elements. Each element is a list of
  \code{GRanges} for each sample. Each GRanges in the list includes the
  location of the CpG sites contained in the reads for each SNP. The GRanges
  metadata table contains: 
  \describe{
\item{\code{cov.ref}}{Number of reads of "reference" allele in that SNP}
\item{\code{cov.alt}}{Number of reads of "alternative" allele in that SNP}
\item{\code{meth.ref}}{Number of methylated reads of "reference" allele in
that SNP}
\item{\code{cov.ref}}{Number of methylated reads of "alternative" allele in
that SNP}
\item{\code{snp}}{The SNP containing the reads}
}

For further details, 
see \url{https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6949/}
sample names in in ArrayExpress do not necessarily match names given here!
}
\usage{
extractbams_output
}
\description{
4 Patients from a previous study (Parker et al, 2018.) with colorectal cancer
were sequenced and the normal and cancerous tissue of each patient was
obtained. The data includes a subset of chromosome 19.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_derivedasm.R
\name{calc_derivedasm}
\alias{calc_derivedasm}
\title{Calculate SNP-based ASM}
\usage{
calc_derivedasm(sampleList, cores = 1, verbose = TRUE)
}
\arguments{
\item{sampleList}{List of samples returned from \code{\link{extract_bams}}.}

\item{cores}{Number of cores to thread.}

\item{verbose}{If the function should be verbose.}
}
\value{
\code{RangedSummarizedExperiment} containing in assays:
\itemize{
\item der.ASM: matrix with SNP-based ASM
\item snp.table: Matrix with SNP associated to the CpG site.
\item ref.cov: Coverage of the 'reference' allele.
\item alt.cov: Coevarage of the 'alternative' allele.
\item ref.meth: Methylated reads from the 'reference' allele.
\item alt.meth: Methylated reads from the 'alternative' allele.
}
}
\description{
Combines all the \code{GRangeslist} generated in \code{\link{extract_bams}}
into a \code{\link{RangedSummarizedExperiment}} object, and calculates
SNP-based allele-specific methylation.
}
\examples{
data(extractbams_output)
derASM <- calc_derivedasm(extractbams_output[c(1,2)], cores = 1, 
   verbose = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_dames.R
\name{find_dames}
\alias{find_dames}
\title{Find DAMEs}
\usage{
find_dames(
  sa,
  design,
  coef = 2,
  contrast = NULL,
  smooth = TRUE,
  Q = 0.5,
  pvalAssign = "simes",
  maxGap = 20,
  verbose = TRUE,
  maxPerms = 10,
  method = "ls",
  trend = FALSE,
  ...
)
}
\arguments{
\item{sa}{A \code{SummarizedExperiment} containing ASM values where each row
correspond to a tuple/site and a column to sample/replicate.}

\item{design}{A design matrix created with \code{\link{model.matrix}}.}

\item{coef}{Column in \code{design} specifying the parameter to estimate.
Default = 2.}

\item{contrast}{a contrast matrix, generated with
\code{\link{makeContrasts}}.}

\item{smooth}{Whether smoothing should be applied to the t-Statistics.
Default = TRUE.}

\item{Q}{The percentile set to get a cutoff value K. K is the value on the
Qth quantile of the absolute values of the given (smoothed) t-statistics.
Only necessary if \code{pvalAssign} = 'empirical'. Default = 0.5.}

\item{pvalAssign}{Choose method to assign pvalues, either 'simes' (default)
or 'empirical'. This second one performs \code{maxPerms} number of
permutations to calculate null statistics, and runs \code{regionFinder}.}

\item{maxGap}{Maximum gap between CpGs in a cluster (in bp). NOTE: Regions
can be as small as 1 bp. Default = 20.}

\item{verbose}{If the function should be verbose. Default = TRUE.}

\item{maxPerms}{Maximum possible permutations generated. Only necessary if
\code{pvalAssign} = 'empirical'. Default = 10.}

\item{method}{The method to be used in limma's \code{\link{lmFit}}. The
default is set to 'ls' but can also be set to 'robust', which is
recommended on a real data set.}

\item{trend}{Passed to \code{\link{eBayes}}. Should an intensity-trend be
allowed for the prior variance? Default is that the prior variance is
constant, e.g. FALSE.}

\item{...}{Arguments passed to \code{\link{get_tstats}}.}
}
\value{
A data frame of detected DAMEs ordered by the p-value. Each row
is a DAME and the following information is provided in the columns
(some column names change depending on the \code{pvalAssign} choice):
\itemize{
\item chr: on which chromosome the DAME is found.
\item start: The start position of the DAME.
\item end: The end position of the DAME.
\item pvalSimes: p-value calculated with the Simes method.
\item pvalEmp: Empirical p-value obtained from permuting covariate of interest.
\item sumTstat: Sum of t-stats per segment/cluster.
\item meanTstat: Mean of t-stats per segment/cluster.
\item segmentL: Size of segmented cluster (from \code{\link{getSegments}}).
\item clusterL: Size of original cluster (from \code{\link{clusterMaker}}).
\item FDR: Adjusted p-value using the method of Benjamini, Hochberg. (from
\code{\link{p.adjust}}).
\item numup: Number of sites with ASM increase in cluster (only for Simes).
\item numdown: Number of sites with ASM decrease in cluster (only for Simes).
}
}
\description{
This function finds Differential Allele-specific MEthylated regions (DAMEs).
It uses the \code{\link{regionFinder}} function from \code{bumphunter}, and
asigns p-values either empirically or using the Simes method.
}
\details{
The simes method has higher power to detect DAMEs, but the consistency in
signal across a region is better controlled with the empirical method, since
it uses \code{regionFinder} and \code{getSegments} to find regions with
t-statistics above a cuttof (controled with parameter \code{Q}), whereas
with the 'simes' option, we initially detects clusters of CpG sites/tuples,
and then test if at least 1 differential site/tuple is present in the
cluster.

We recommend trying out different \code{maxGap} and \code{Q} parameters,
since the size and the effect-size of obtained DAMEs change with these
parameters.
}
\examples{
data(readtuples_output)
ASM <- calc_asm(readtuples_output)
grp <- factor(c(rep('CRC',3),rep('NORM',2)), levels = c('NORM', 'CRC'))
mod <- model.matrix(~grp)
dames <- find_dames(ASM, mod, verbose = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_asm.R
\name{modulus_sqrt}
\alias{modulus_sqrt}
\title{Get Modulus Square Root}
\usage{
modulus_sqrt(values)
}
\arguments{
\item{values}{Vector or matrix of ASM scores where each column is a sample.
These values are transformed with a square root transformation that
(doesn't) preserve the sign.}
}
\value{
Vector or matrix of transformed scores.
}
\description{
Function to calculate signed square root (aka modulus square root).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methyl_circle_plot.R
\name{methyl_circle_plot}
\alias{methyl_circle_plot}
\title{Draw methylation circle plot}
\usage{
methyl_circle_plot(
  snp,
  vcfFile,
  bamFile,
  refFile,
  build = "hg19",
  dame = NULL,
  letterSize = 2.5,
  pointSize = 3,
  sampleName = "sample1",
  cpgsite = NULL,
  sampleReads = FALSE,
  numReads = 20
)
}
\arguments{
\item{snp}{GRanges object containing SNP location.}

\item{vcfFile}{vcf file.}

\item{bamFile}{bismark bam file path.}

\item{refFile}{fasta reference file path. Or \code{DNAStringSet} with DNA
sequence.}

\item{build}{genome build used. default = "hg19"}

\item{dame}{(optional) GRanges object containing a region to plot.}

\item{letterSize}{Size of alleles drawn in plot. Default = 2.5.}

\item{pointSize}{Size of methylation circles. Default = 3.}

\item{sampleName}{FIX?: this is to save the vcf file to not generate it every
time you run the function.}

\item{cpgsite}{(optional) GRanges object containing a single CpG site
location of interest.}

\item{sampleReads}{Whether a subset of reads should be plotted.
Default = FALSE.}

\item{numReads}{Number of reads to plot per allele, if sampleReads is TRUE.
Default = 20}
}
\value{
Plot
}
\description{
Draws CpG site methylation status as points, in reads containing a specific
SNP. Generates one plot per bam file.
}
\examples{
DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')

get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
bam_files <- get_data_path('NORM1_chr19_trim.bam')
vcf_files <- get_data_path('NORM1.chr19.trim.vcf')
sample_names <- 'NORM1'
#reference_file
suppressPackageStartupMessages({library(BSgenome.Hsapiens.UCSC.hg19)})
genome <- BSgenome.Hsapiens.UCSC.hg19
seqnames(genome) <- gsub("chr","",seqnames(genome))
dna <- DNAStringSet(genome[[19]], use.names = TRUE)
names(dna) <- 19

snp <- GenomicRanges::GRanges(19, IRanges::IRanges(292082, width = 1))
methyl_circle_plot(snp = snp,
 vcfFile = vcf_files,
 bamFile = bam_files,
 refFile = dna,
 sampleName = sample_names)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/empirical_pval.R
\name{empirical_pval}
\alias{empirical_pval}
\title{Calculate empirical region-level p-value}
\usage{
empirical_pval(
  presa,
  design,
  rforiginal,
  coeff,
  cont,
  smooth,
  maxPerms = 10,
  Q,
  maxGap,
  method,
  ...
)
}
\arguments{
\item{presa}{SExperiment output from \code{calc_derivedasm} or
\code{calc_asm}.}

\item{design}{design matrix.}

\item{rforiginal}{data.frame of DAMEs calculated with original design.}

\item{coeff}{Coefficient of interest to permute.}

\item{cont}{same as in \code{get_tstats}.}

\item{smooth}{Boolean.}

\item{maxPerms}{Maximum possible permutations generated. Default = 10.}

\item{Q}{Quantile for cuttof.}

\item{maxGap}{Same as other functions in the package.}

\item{method}{lmFit method.}

\item{...}{Passed to \code{get_tstats} and then to \code{loessByCluster}.}
}
\value{
Vector of empirical p-values.
}
\description{
This function permutes the coefficient of interest and re-runs
\code{\link{get_tstats}} and \code{\link{regionFinder}} for each permutation.
Code for permutations copied from the \code{dmrseq} function from the package
of the same name.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simes_pval.R
\name{simes_pval}
\alias{simes_pval}
\title{Calculate region-level p-value}
\usage{
simes_pval(sat, smtstat, midpt)
}
\arguments{
\item{sat}{Output from \code{\link{get_tstats}}.}

\item{smtstat}{(Smoothed) tstat vector from \code{\link{get_tstats}}.}

\item{midpt}{Coordinate vector for each CpG site/tuple.}
}
\value{
Vector of summarized pvals
}
\description{
This function uses the Simes method to calculate a regional-level p-value
based on the single-eBayes p-values. It highly depends on the choice of
\code{maxGap} in \code{find_dames}.
}
\details{
When used as a FDR-control method, for positively correlated P-values, Simes
method is even closer to the nominal alpha level than the Bonferroni-Holm
method.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split_bams.R
\name{extract_bams}
\alias{extract_bams}
\title{Detect allele-specific methylation from a bam file}
\usage{
extract_bams(
  bamFiles,
  vcfFiles,
  sampleNames,
  referenceFile,
  coverage = 4,
  cores = 1,
  verbose = TRUE
)
}
\arguments{
\item{bamFiles}{List of bam files.}

\item{vcfFiles}{List of vcf files.}

\item{sampleNames}{Names of files in the list.}

\item{referenceFile}{fasta file used to generate the bam files. Or 
\code{DNAStringSet} with DNA sequence.}

\item{coverage}{Minimum number of reads covering a CpG site on each allele.
Default = 2.}

\item{cores}{Number of cores to use. See package {parallel} for description
of core. Default = 1.}

\item{verbose}{Default = TRUE}
}
\value{
A list of GRanges for each sample. Each list is saved in a separate
  .rds file.
}
\description{
The function takes a bam (from bismark) and vcf file for each sample. For
each SNP contained in the vcfile it calculates the proportion of methylated
reads for each CpG site at each allele. At the end it returns (saves to
working directory) a GRanges list, where each GRanges contains all the CpG
sites overlapping the reads containing a specific SNP.
}
\examples{
DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)
bamFiles <- get_data_path('NORM1_chr19_trim.bam')
vcfFiles <- get_data_path('NORM1.chr19.trim.vcf')
sampleNames <- 'NORM1'

#referenceFile 
suppressPackageStartupMessages({library(BSgenome.Hsapiens.UCSC.hg19)})
genome <- BSgenome.Hsapiens.UCSC.hg19
seqnames(genome) <- gsub("chr","",seqnames(genome))
dna <- DNAStringSet(genome[[19]], use.names = TRUE)
names(dna) <- 19

GRanges_list <- extract_bams(bamFiles, vcfFiles, sampleNames, dna)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_tstats.R
\name{get_tstats}
\alias{get_tstats}
\title{Get t-Statistics}
\usage{
get_tstats(
  sa,
  design,
  contrast = NULL,
  method = "ls",
  trend = FALSE,
  smooth = FALSE,
  maxGap = 20,
  coef = 2,
  verbose = TRUE,
  filter = TRUE,
  ...
)
}
\arguments{
\item{sa}{A SummarizedExperiment containing ASM values where each row and
column correspond to a tuple/site and sample respectively.}

\item{design}{a design matrix created with \code{\link{model.matrix}}.}

\item{contrast}{a contrast matrix, generated with
\code{\link{makeContrasts}}.}

\item{method}{The method to be used in limma's \code{\link{lmFit}}. The
default is set to 'ls' but can also be set to 'robust', which is
recommended on a real data set.}

\item{trend}{Passed to \code{\link{eBayes}}. Should an intensity-trend be
allowed for the prior variance? Default is that the prior variance is
constant, e.g. FALSE.}

\item{smooth}{Whether smoothing should be applied to the t-Statistics.
Default = FALSE. If TRUE, wherever smoothing is not possible, the
un-smoothed t-stat is used instead.}

\item{maxGap}{The maximum allowed gap between genomic positions for
clustering of genomic regions to be used in smoothing. Default = 20.}

\item{coef}{Column in model.matrix specifying the parameter to estimate.
Default = 2. If \code{contrast} specified, column with contrast of interest.}

\item{verbose}{Set verbose. Default = TRUE.}

\item{filter}{Remove empty tstats. Default = TRUE.}

\item{...}{Arguments passed to \code{\link{loessByCluster}}. Only used if
\code{smooth} = TRUE.}
}
\value{
A vector of t-Statistics within the
  \code{RangedSummarizedExperiment}.
}
\description{
This function calculates a moderated t-Statistic per site or tuple using
\code{limma}'s \code{\link{lmFit}} and \code{\link{eBayes}} functions. It
then smoothes the obtained t-Statistics using \code{bumphunter}'s
\code{\link{smoother}} function.
}
\details{
The smoothing is done on genomic clusters consisting of CpGs that are close
to each other. In the case of tuples, the midpoint of the two genomic
positions in each tuple is used as the genomic position of that tuple, to
perform the smoothing.The function takes a \code{RangedSummarizedExperiment}
generated by \code{\link{calc_derivedasm}} or \code{\link{calc_asm}}
containing ASM across samples, and the index of control and treatment
samples.
}
\examples{
data(readtuples_output)
ASM <- calc_asm(readtuples_output)
grp <- factor(c(rep('CRC',3),rep('NORM',2)), levels = c('NORM', 'CRC'))
mod <- model.matrix(~grp)
tstats <- get_tstats(ASM, mod)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_tuples.R
\name{read_tuples}
\alias{read_tuples}
\title{Read in list of methtuple files}
\usage{
read_tuples(files, sampleNames, minCoverage = 2, maxGap = 20, verbose = TRUE)
}
\arguments{
\item{files}{List of methtuple files.}

\item{sampleNames}{Names of files in the list.}

\item{minCoverage}{The minimum coverage per tuple. Tuples with a coverage <
minCoverage are filtered out. Default = 2.}

\item{maxGap}{The maximum allowed distance between two positions in a tuple.
Only distances that are <= maxGap are kept. Default = 150 base pairs.}

\item{verbose}{If the function should be verbose.}
}
\value{
A list of data frames, where each data frame corresponds to one file.
}
\description{
This function reads in a list of files obtained from the methtuple tool. It
filters out tuples based on the set minimum coverage (min_cov) and the
maximum allowed distance (maxGap) between two genomic positions in a tuple.
}
\examples{
DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)

tuple_files <- list.files(DATA_PATH_DIR, '.tsv.gz')
tuple_files <- get_data_path(tuple_files)
ASM <- read_tuples(tuple_files, c('CRC1', 'NORM1'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methyl_MDS_plot.R
\name{methyl_MDS_plot}
\alias{methyl_MDS_plot}
\title{Multidimensional scaling plot of distances between methylation 
proportions (beta values)}
\usage{
methyl_MDS_plot(x, group, top = 1000, coverage = 5, adj = 0.02, pointSize = 4)
}
\arguments{
\item{x}{\code{RangedSummarizedExperiment}, output from
\code{calc_derivedasm} or \code{calc_asm}.}

\item{group}{Vector of group or any other labels, same length as number of
samples.}

\item{top}{Number of top CpG sites used to calculate pairwise distances.}

\item{coverage}{Minimum number of reads covering a CpG site on each allele.
Default = 5.}

\item{adj}{Text adjustment in y-axis. Default = 0.2.}

\item{pointSize}{Default = 4.}
}
\value{
Two-dimensional MDS plot.
}
\description{
Same as \code{\link{plotMDS}}, except for an arc-sine transformation of the
methylation proportions.
}
\examples{
data(readtuples_output)
ASM <- calc_asm(readtuples_output)
grp <- factor(c(rep('CRC',3),rep('NORM',2)), levels = c('NORM', 'CRC'))
methyl_MDS_plot(ASM, grp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_asm.R
\name{calc_asm}
\alias{calc_asm}
\title{Calculate ASM Score}
\usage{
calc_asm(
  sampleList,
  beta = 0.5,
  a = 0.2,
  transform = modulus_sqrt,
  coverage = 5,
  verbose = TRUE
)
}
\arguments{
\item{sampleList}{List of samples returned from \code{\link{read_tuples}}}

\item{beta}{The beta parameter used to calculate the weight in the ASM score.
\code{link{calc_weight}} uses this parameter to penalize fully methylated
or unmethylated tuples. Default = 0.5.}

\item{a}{The distance from 0.5 allowed, where 0.5 is a perfect MM:UU balance
for a tuple. In the default mode this value is set to 0.2, and we account
for the instances where the balance is between 0.3 and 0.7.}

\item{transform}{Transform the calculated tuple ASM scores. We use the
modulus square root function which outputs the square root, while
preserving the original sign.}

\item{coverage}{Remove tuples with total reads below coverage. Default
= 5.}

\item{verbose}{If the function should be verbose. Default = TRUE.}
}
\value{
A \code{SummarizedExperiment} of ASM scores where the rows are all
  the tuples and the columns the sample names.
}
\description{
This function takes in a list of samples resulting from the read_tuples
function and returns a SummarizedExperiment of Allele-Specific Methylation
(ASM) scores, where each row is a tuple and each column is a sample.
}
\details{
Calculates ASM score for a list of samples in the output format of the result
of read_tuples This functions uses the following other functions: process,
calcScore, calcWeight.
}
\examples{
DATA_PATH_DIR <- system.file('extdata', '.', package = 'DAMEfinder')
get_data_path <- function(file_name) file.path(DATA_PATH_DIR, file_name)

tuple_files <- list.files(DATA_PATH_DIR, '.tsv.gz')
tuple_files <- get_data_path(tuple_files)
ASM <- read_tuples(tuple_files, c('CRC1', 'NORM1'))
ASMscore <- calc_asm(ASM)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score_tracks.R
\name{dame_track}
\alias{dame_track}
\title{Plot score tracks}
\usage{
dame_track(
  dame,
  window = 0,
  positions = 0,
  derASM = NULL,
  ASM = NULL,
  colvec = NULL,
  plotSNP = FALSE
)
}
\arguments{
\item{dame}{GRanges object containing a region of interest, or detected with
find_dames}

\item{window}{Number of CpG sites outside (up or down-stream) of the DAME
should be plotted. Default = 0.}

\item{positions}{Number of bp sites outside (up or down-stream) of the DAME
should be plotted. Default = 0.}

\item{derASM}{SummarizedExperiment object obtained from calc_derivedasm
(Filtering should be done by the user)}

\item{ASM}{SummarizedExperiment object obtained from calc_asm (Filtering
should be done by the user)}

\item{colvec}{Vector of colors (mainly useful for the SNP plot, because I add
it with cowplot, so I don't export a ggplot, optional)}

\item{plotSNP}{whether to add the SNP track, only if derASM is specified.
Default = FALSE}
}
\value{
Plot
}
\description{
Plot score tracks
}
\examples{
library(GenomicRanges)
DAME <- GRanges(19, IRanges(306443,310272))
data('readtuples_output')
ASM <- calc_asm(readtuples_output)
SummarizedExperiment::colData(ASM)$group <- c(rep('CRC',3),rep('NORM',2))
SummarizedExperiment::colData(ASM)$samples <- colnames(ASM)
dame_track(dame = DAME, 
           ASM = ASM) 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split_reads.R
\name{splitReads}
\alias{splitReads}
\title{Divide read names by allele}
\usage{
splitReads(alns, v, snp)
}
\arguments{
\item{alns}{GenomicAlignments object.}

\item{v}{Nucleotide of reference (or alternative) allele.}

\item{snp}{GRanges object containing SNP location.}
}
\value{
A named list of vectors, each vector containing read names for each
  allele.
}
\description{
Takes a GenomicAlignments object and returns a list of read names dividied by
allele.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{readtuples_output}
\alias{readtuples_output}
\title{read_tuples() output.}
\format{
A large list with 5 elements. Each element is a \code{tibble} with
  the coordinates of the pairs of CpG sites (tuples). Rest of the tibble
  contains:
  \describe{
\item{\code{MM}}{Number of reads with both CpG sites methylated}
\item{\code{MU}}{Number of reads with first CpG site methylated}
\item{\code{UM}}{Number of reads with second CpG site methylated}
\item{\code{UU}}{Number of reads with both CpG sites unmethylated}
\item{\code{cov}}{Coverage, total reads at tuple}
\item{\code{inter_dist}}{Distance in bp between CpG sites}
}

For further details, 
see \url{https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6949/}
sample names in in ArrayExpress do not necessarily match names given here!
}
\usage{
readtuples_output
}
\description{
3 Patients from a previous study (Parker et al, 2018.) with colorectal cancer
were sequenced and the normal and cancerous tissue of each patient was
obtained. The data includes a subset of chromosome 19. Here one normal sample
is not included.
}
\keyword{datasets}
